@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -48,6 +48,10 @@ typedef struct _FolderPrivData FolderPrivData;
 struct _FolderPrivData {
 	Folder *folder;
 	FolderItem *junk;
+
+	FolderUIFunc2 ui_func2;
+	gpointer ui_func2_data;
+
 	gpointer data;
 };
 
@@ -442,6 +446,42 @@ void folder_set_ui_func(Folder *folder, FolderUIFunc func, gpointer data)
 	folder->ui_func_data = data;
 }
 
+void folder_set_ui_func2(Folder *folder, FolderUIFunc2 func, gpointer data)
+{
+	FolderPrivData *priv;
+
+	priv = folder_get_priv(folder);
+	if (priv) {
+		priv->ui_func2 = func;
+		priv->ui_func2_data = data;
+	}
+}
+
+FolderUIFunc2 folder_get_ui_func2(Folder *folder)
+{
+	FolderPrivData *priv;
+
+	priv = folder_get_priv(folder);
+	if (priv)
+		return priv->ui_func2;
+
+	return NULL;
+}
+
+gboolean folder_call_ui_func2(Folder *folder, FolderItem *item, guint count,
+			      guint total)
+{
+	FolderPrivData *priv;
+
+	priv = folder_get_priv(folder);
+	if (priv && priv->ui_func2) {
+		return priv->ui_func2(folder, item, count, total,
+				      priv->ui_func2_data);
+	}
+
+	return TRUE;
+}
+
 void folder_set_name(Folder *folder, const gchar *name)
 {
 	g_return_if_fail(folder != NULL);
@@ -1105,21 +1145,25 @@ gchar *folder_get_path(Folder *folder)
 			path = path_;
 		}
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
+		gchar *server;
 		gchar *uid;
 
 		g_return_val_if_fail(folder->account != NULL, NULL);
+		server = uriencode_for_filename(folder->account->recv_server);
 		uid = uriencode_for_filename(folder->account->userid);
 		path = g_strconcat(get_imap_cache_dir(),
-				   G_DIR_SEPARATOR_S,
-				   folder->account->recv_server,
+				   G_DIR_SEPARATOR_S, server,
 				   G_DIR_SEPARATOR_S, uid, NULL);
 		g_free(uid);
+		g_free(server);
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
+		gchar *server;
+
 		g_return_val_if_fail(folder->account != NULL, NULL);
+		server = uriencode_for_filename(folder->account->nntp_server);
 		path = g_strconcat(get_news_cache_dir(),
-				   G_DIR_SEPARATOR_S,
-				   folder->account->nntp_server,
-				   NULL);
+				   G_DIR_SEPARATOR_S, server, NULL);
+		g_free(server);
 	} else
 		path = NULL;
 
@@ -1656,6 +1700,8 @@ static gboolean folder_build_tree(GNode *node, gpointer data)
 				qsearch_cond_type = 7;
 			else if (!strcmp(attr->value, "in-addressbook"))
 				qsearch_cond_type = 8;
+			else if (!strcmp(attr->value, "last30"))
+				qsearch_cond_type = 9;
 		} else if (!strcmp(attr->name, "account_id")) {
 			account = account_find_from_id(atoi(attr->value));
 			if (!account) g_warning("account_id: %s not found\n",
@@ -1834,7 +1880,7 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 					"mark", "unread", "mime", "to"};
 	static gchar *qsearch_cond_str[] = {"all", "unread", "mark", "clabel",
 					    "mime", "w1day", "last5", "last7",
-					    "in-addressbook"};
+					    "in-addressbook", "last30"};
 
 	g_return_if_fail(node != NULL);
 	g_return_if_fail(fp != NULL);
@@ -1893,7 +1939,7 @@ static void folder_write_list_recursive(GNode *node, gpointer data)
 				fprintf(fp, " sort_type=\"descending\"");
 		}
 		if (item->qsearch_cond_type > 0 &&
-		    item->qsearch_cond_type < 9) {
+		    item->qsearch_cond_type < 10) {
 			fprintf(fp, " qsearch_cond=\"%s\"",
 				qsearch_cond_str[item->qsearch_cond_type]);
 		}
