@@ -1229,6 +1229,7 @@ static gint mh_create_tree(Folder *folder)
 	MAKE_DIR_IF_NOT_EXIST(QUEUE_DIR);
 	MAKE_DIR_IF_NOT_EXIST(DRAFT_DIR);
 	MAKE_DIR_IF_NOT_EXIST(TRASH_DIR);
+	MAKE_DIR_IF_NOT_EXIST(JUNK_DIR);
 
 	return 0;
 }
@@ -1664,6 +1665,8 @@ static void mh_remove_missing_folder_items(Folder *folder)
 			mh_remove_missing_folder_items_func, folder);
 }
 
+#define MAX_RECURSION_LEVEL	64
+
 static void mh_scan_tree_recursive(FolderItem *item)
 {
 	Folder *folder;
@@ -1690,6 +1693,16 @@ static void mh_scan_tree_recursive(FolderItem *item)
 
 	folder = item->folder;
 
+	if (g_node_depth(item->node) >= MAX_RECURSION_LEVEL) {
+		g_warning("mh_scan_tree_recursive(): max recursion level (%u) reached.", MAX_RECURSION_LEVEL);
+		return;
+	}
+
+	debug_print("scanning %s ...\n",
+		    item->path ? item->path : LOCAL_FOLDER(folder)->rootpath);
+	if (folder->ui_func)
+		folder->ui_func(folder, item, folder->ui_func_data);
+
 	fs_path = item->path ?
 		g_filename_from_utf8(item->path, -1, NULL, NULL, NULL)
 		: g_strdup(".");
@@ -1712,12 +1725,6 @@ static void mh_scan_tree_recursive(FolderItem *item)
 #endif
 	g_free(fs_path);
 
-	debug_print("scanning %s ...\n",
-		    item->path ? item->path
-		    : LOCAL_FOLDER(item->folder)->rootpath);
-	if (folder->ui_func)
-		folder->ui_func(folder, item, folder->ui_func_data);
-
 #ifdef G_OS_WIN32
 	do {
 		if (!wfd.file_name) continue;
@@ -1820,6 +1827,10 @@ static void mh_scan_tree_recursive(FolderItem *item)
 					   !strcmp(dir_name, TRASH_DIR)) {
 					new_item->stype = F_TRASH;
 					folder->trash = new_item;
+				} else if (!folder_get_junk(folder) &&
+					   !strcmp(dir_name, JUNK_DIR)) {
+					new_item->stype = F_JUNK;
+					folder_set_junk(folder, new_item);
 				}
 			}
 
