@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -51,6 +51,10 @@ static PrefParam param[] = {
 	 &prefs_common.enable_newmsg_notify_sound, P_BOOL},
 	{"newmsg_notify_sound", NULL, &prefs_common.newmsg_notify_sound,
 	 P_STRING},
+	{"enable_newmsg_notify_window", "TRUE",
+	 &prefs_common.enable_newmsg_notify_window, P_BOOL},
+	{"notify_window_period", "10",
+	 &prefs_common.notify_window_period, P_INT},
 
 	{"inc_local", "FALSE", &prefs_common.inc_local, P_BOOL},
 	{"filter_on_inc_local", "TRUE", &prefs_common.filter_on_inc, P_BOOL},
@@ -329,8 +333,13 @@ static PrefParam param[] = {
 	{"display_header_pane", "TRUE", &prefs_common.display_header_pane,
 	 P_BOOL},
 	{"show_attach_tab", "FALSE", &prefs_common.show_attach_tab, P_BOOL},
+	{"show_attached_files_first", "TRUE",
+	 &prefs_common.show_attached_files_first, P_BOOL},
+	{"attach_toolbutton_pos", "0", &prefs_common.attach_toolbutton_pos,
+	 P_INT},
 	{"display_header", "TRUE", &prefs_common.display_header, P_BOOL},
 	{"render_html", "TRUE", &prefs_common.render_html, P_BOOL},
+	{"alt_prefer_html", "FALSE", &prefs_common.alt_prefer_html, P_BOOL},
 	{"html_only_as_attach", "FALSE", &prefs_common.html_only_as_attach,
 	 P_BOOL},
 	{"line_space", "2", &prefs_common.line_space, P_INT},
@@ -387,6 +396,8 @@ static PrefParam param[] = {
 	 P_BOOL},
 	{"delete_junk_on_receive", "TRUE", &prefs_common.delete_junk_on_recv,
 	 P_BOOL},
+	{"nofilter_junk_sender_in_book", "TRUE",
+	 &prefs_common.nofilter_junk_sender_in_book, P_BOOL},
 	{"mark_junk_as_read", "FALSE", &prefs_common.mark_junk_as_read, P_BOOL},
 
 	/* Privacy */
@@ -506,6 +517,7 @@ static PrefParam param[] = {
 	{"filesel_prev_open_dir", NULL, &prefs_common.prev_open_dir, P_STRING},
 	{"filesel_prev_save_dir", NULL, &prefs_common.prev_save_dir, P_STRING},
 	{"filesel_prev_folder_dir", NULL, &prefs_common.prev_folder_dir, P_STRING},
+	{"filesel_save_file_type", "0", &prefs_common.save_file_type, P_INT},
 
 	{NULL, NULL, NULL, P_OTHER}
 };
