@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -688,9 +688,14 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 		case H_CONTENT_TYPE:
 			if (!g_ascii_strncasecmp(hp, "multipart", 9)) {
 				MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME);
-			} else if (!charset) {
-				procmime_scan_content_type_str
-					(hp, NULL, &charset, NULL, NULL);
+			} else {
+				if (!g_ascii_strncasecmp(hp, "text/html", 9)) {
+					MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME_HTML);
+				}
+				if (!charset) {
+					procmime_scan_content_type_str
+						(hp, NULL, &charset, NULL, NULL);
+				}
 			}
 			break;
 		case H_SEEN:
@@ -740,6 +745,28 @@ MsgInfo *procheader_parse_stream(FILE *fp, MsgFlags flags, gboolean full)
 		msginfo->inreplyto =
 			g_strdup((gchar *)msginfo->references->data);
 
+	if (MSG_IS_MIME(msginfo->flags)) {
+		MimeInfo *mimeinfo, *part;
+		gboolean has_html = FALSE;
+
+		part = mimeinfo = procmime_scan_message_stream(fp);
+		while (part) {
+			if (part->mime_type != MIME_TEXT &&
+			    part->mime_type != MIME_TEXT_HTML &&
+			    part->mime_type != MIME_MULTIPART)
+				break;
+			if (part->mime_type == MIME_TEXT_HTML)
+				has_html = TRUE;
+			part = procmime_mimeinfo_next(part);
+		}
+
+		if (has_html && !part) {
+			MSG_SET_TMP_FLAGS(msginfo->flags, MSG_MIME_HTML);
+		}
+
+		procmime_mimeinfo_free_all(mimeinfo);
+	}
+
 	g_free(charset);
 
 	return msginfo;
@@ -749,7 +776,7 @@ gchar *procheader_get_fromname(const gchar *str)
 {
 	gchar *tmp, *name;
 
-	Xstrdup_a(tmp, str, return NULL);
+	tmp = g_strdup(str);
 
 	if (*tmp == '\"') {
 		extract_quote_with_escape(tmp, '\"');
@@ -767,10 +794,11 @@ gchar *procheader_get_fromname(const gchar *str)
 		g_strstrip(tmp);
 	}
 
-	if (*tmp == '\0')
+	if (*tmp == '\0') {
+		g_free(tmp);
 		name = g_strdup(str);
-	else
-		name = g_strdup(tmp);
+	} else
+		name = tmp;
 
 	return name;
 }
@@ -808,44 +836,39 @@ static gint procheader_scan_date_string(const gchar *str,
 {
 	gint result;
 
+	*zone = '\0';
 	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d %5s",
 			weekday, day, month, year, hh, mm, ss, zone);
-	if (result == 8) return 0;
+	if (result >= 7) return 0;
 
 	result = sscanf(str, "%3s,%d %9s %d %2d:%2d:%2d %5s",
 			weekday, day, month, year, hh, mm, ss, zone);
-	if (result == 8) return 0;
+	if (result >= 7) return 0;
+
+	result = sscanf(str, "%3s,%d %9s %d %2d.%2d.%2d %5s",
+			weekday, day, month, year, hh, mm, ss, zone);
+	if (result >= 7) return 0;
+
+	result = sscanf(str, "%3s %d, %9s %d %2d:%2d:%2d %5s",
+			weekday, day, month, year, hh, mm, ss, zone);
+	if (result >= 7) return 0;
 
 	result = sscanf(str, "%d %9s %d %2d:%2d:%2d %5s",
 			day, month, year, hh, mm, ss, zone);
-	if (result == 7) return 0;
+	if (result >= 6) return 0;
 
-	*zone = '\0';
-	result = sscanf(str, "%10s %d %9s %d %2d:%2d:%2d",
-			weekday, day, month, year, hh, mm, ss);
-	if (result == 7) return 0;
-
-	result = sscanf(str, "%d %9s %d %2d:%2d:%2d",
-			day, month, year, hh, mm, ss);
+	result = sscanf(str, "%d-%2s-%2d %2d:%2d:%2d",
+			year, month, day, hh, mm, ss);
 	if (result == 6) return 0;
 
 	*ss = 0;
 	result = sscanf(str, "%10s %d %9s %d %2d:%2d %5s",
 			weekday, day, month, year, hh, mm, zone);
-	if (result == 7) return 0;
+	if (result >= 6) return 0;
 
 	result = sscanf(str, "%d %9s %d %2d:%2d %5s",
 			day, month, year, hh, mm, zone);
-	if (result == 6) return 0;
-
-	*zone = '\0';
-	result = sscanf(str, "%10s %d %9s %d %2d:%2d",
-			weekday, day, month, year, hh, mm);
-	if (result == 6) return 0;
-
-	result = sscanf(str, "%d %9s %d %2d:%2d",
-			day, month, year, hh, mm);
-	if (result == 5) return 0;
+	if (result >= 5) return 0;
 
 	return -1;
 }
@@ -867,6 +890,7 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 	if (procheader_scan_date_string(src, weekday, &day, month, &year,
 					&hh, &mm, &ss, zone) < 0) {
+		g_warning("procheader_scan_date_string: date parse failed: %s", src);
 		if (dest && len > 0)
 			strncpy2(dest, src, len);
 		return 0;
@@ -881,10 +905,14 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	}
 
 	month[3] = '\0';
-	for (p = monthstr; *p != '\0'; p += 3) {
-		if (!g_ascii_strncasecmp(p, month, 3)) {
-			dmonth = (gint)(p - monthstr) / 3 + 1;
-			break;
+	if (g_ascii_isdigit(month[0])) {
+		dmonth = atoi(month);
+	} else {
+		for (p = monthstr; *p != '\0'; p += 3) {
+			if (!g_ascii_strncasecmp(p, month, 3)) {
+				dmonth = (gint)(p - monthstr) / 3 + 1;
+				break;
+			}
 		}
 	}
 
@@ -900,14 +928,22 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 
 	timer = mktime(&t);
 	if (timer == -1) {
-		if (dest)
-			dest[0] = '\0';
-		return 0;
+		if (year >= 2038) {
+			g_warning("mktime: date overflow: %s", src);
+			timer = G_MAXINT - 12 * 3600;
+		} else {
+			g_warning("mktime: can't convert date: %s", src);
+			if (dest)
+				dest[0] = '\0';
+			return 0;
+		}
 	}
 
-	tz_offset = remote_tzoffset_sec(zone);
-	if (tz_offset != -1)
-		timer += tzoffset_sec(&timer) - tz_offset;
+	if (timer < G_MAXINT - 12 * 3600) {
+		tz_offset = remote_tzoffset_sec(zone);
+		if (tz_offset != -1)
+			timer += tzoffset_sec(&timer) - tz_offset;
+	}
 
 	if (dest)
 		procheader_date_get_localtime(dest, len, timer);
@@ -919,9 +955,8 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 {
 	struct tm *lt;
 	gchar *default_format = "%y/%m/%d(%a) %H:%M";
-	gchar *tmp, *buf;
-
-	Xalloca(tmp, len + 1, dest[0] = '\0'; return;);
+	gchar *buf;
+	gchar tmp[BUFFSIZE];
 
 	lt = localtime(&timer);
 	if (!lt) {
@@ -931,9 +966,9 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 	}
 
 	if (prefs_common.date_format)
-		strftime(tmp, len, prefs_common.date_format, lt);
+		strftime(tmp, sizeof(tmp), prefs_common.date_format, lt);
 	else
-		strftime(tmp, len, default_format, lt);
+		strftime(tmp, sizeof(tmp), default_format, lt);
 
 	buf = conv_localetodisp(tmp, NULL);
 	strncpy2(dest, buf, len);
