@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1720,100 +1720,90 @@ static gint print_command_exec(const gchar *file, const gchar *cmdline)
 	if (buf[strlen(buf) - 1] != '&')
 		strcat(buf, "&");
 
-	system(buf);
+	if (system(buf) != 0)
+		return -1;
 
 	return 0;
 }
 
+static void procmsg_write_headers(MsgInfo *msginfo, MimeInfo *partinfo,
+				  FILE *fp, FILE *dest_fp,
+				  const gchar *encoding, gboolean all_headers)
+{
+	GPtrArray *headers;
+	gint i;
+
+	if (all_headers)
+		headers = procheader_get_header_array_asis(fp, NULL);
+	else
+		headers = procheader_get_header_array_for_display(fp, NULL);
+
+	for (i = 0; i < headers->len; i++) {
+		Header *hdr;
+		gchar *file_str;
+		const gchar *body;
+
+		hdr = g_ptr_array_index(headers, i);
+
+		if (partinfo) {
+			if (!g_ascii_strcasecmp(hdr->name, "Subject") ||
+			    !g_ascii_strcasecmp(hdr->name, "From") ||
+			    !g_ascii_strcasecmp(hdr->name, "To") ||
+			    !g_ascii_strcasecmp(hdr->name, "Cc")) {
+				unfold_line(hdr->body);
+			}
+
+			body = hdr->body;
+			while (g_ascii_isspace(*body))
+				body++;
+		} else {
+			if (!g_ascii_strcasecmp(hdr->name, "Subject"))
+				body = msginfo->subject;
+			else if (!g_ascii_strcasecmp(hdr->name, "From"))
+				body = msginfo->from;
+			else if (!g_ascii_strcasecmp(hdr->name, "To"))
+				body = msginfo->to;
+			else if (!g_ascii_strcasecmp(hdr->name, "Cc")) {
+				unfold_line(hdr->body);
+				body = hdr->body;
+				while (g_ascii_isspace(*body))
+					body++;
+			} else {
+				body = hdr->body;
+				while (g_ascii_isspace(*body))
+					body++;
+			}
+		}
+
+		if (body && *body != '\0') {
+			file_str = conv_codeset_strdup
+				(body, CS_INTERNAL, encoding);
+			fprintf(dest_fp, "%s: %s\n", hdr->name,
+				file_str ? file_str : body);
+			g_free(file_str);
+		} else {
+			fprintf(dest_fp, "%s: (none)\n", hdr->name);
+		}
+	}
+
+	procheader_header_array_destroy(headers);
+}
+
 void procmsg_print_message(MsgInfo *msginfo, const gchar *cmdline,
 			   gboolean all_headers)
 {
 	gchar *prtmp;
-	FILE *msgfp, *tmpfp, *prfp;
-	GPtrArray *headers;
-	gint i;
-	gchar buf[BUFFSIZE];
 
 	g_return_if_fail(msginfo != NULL);
 
-	if ((tmpfp = procmime_get_first_text_content
-		(msginfo, conv_get_locale_charset_str())) == NULL) {
-		g_warning("Can't get text part\n");
-		return;
-	}
-
 	prtmp = g_strdup_printf("%s%cprinttmp-%08x.txt",
 				get_mime_tmp_dir(), G_DIR_SEPARATOR,
 				print_id++);
 
-	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
-		FILE_OP_ERROR(prtmp, "procmsg_print_message: fopen");
-		g_free(prtmp);
-		fclose(tmpfp);
-		return;
-	}
-
-	if ((msgfp = procmsg_open_message(msginfo)) == NULL) {
-		fclose(prfp);
-		g_free(prtmp);
-		fclose(tmpfp);
-		return;
-	}
-
-	if (all_headers)
-		headers = procheader_get_header_array_asis(msgfp, NULL);
-	else
-		headers = procheader_get_header_array_for_display(msgfp, NULL);
-
-	fclose(msgfp);
-
-	for (i = 0; i < headers->len; i++) {
-		Header *hdr;
-		gchar *locale_str;
-		const gchar *body;
-
-		hdr = g_ptr_array_index(headers, i);
-
-		if (!g_ascii_strcasecmp(hdr->name, "Subject"))
-			body = msginfo->subject;
-		else if (!g_ascii_strcasecmp(hdr->name, "From"))
-			body = msginfo->from;
-		else if (!g_ascii_strcasecmp(hdr->name, "To"))
-			body = msginfo->to;
-		else if (!g_ascii_strcasecmp(hdr->name, "Cc")) {
-			unfold_line(hdr->body);
-			body = hdr->body;
-			while (g_ascii_isspace(*body))
-				body++;
-		} else {
-			body = hdr->body;
-			while (g_ascii_isspace(*body))
-				body++;
-		}
-
-		if (body && *body != '\0') {
-			locale_str = conv_codeset_strdup
-				(body, CS_INTERNAL,
-				 conv_get_locale_charset_str());
-			fprintf(prfp, "%s: %s\n", hdr->name,
-				locale_str ? locale_str : body);
-			g_free(locale_str);
-		} else {
-			fprintf(prfp, "%s: (none)\n", hdr->name);
-		}
-	}
-
-	procheader_header_array_destroy(headers);
-
-	fputc('\n', prfp);
-
-	while (fgets(buf, sizeof(buf), tmpfp) != NULL)
-		fputs(buf, prfp);
-
-	fclose(prfp);
-	fclose(tmpfp);
-
-	print_command_exec(prtmp, cmdline);
+	if (procmsg_save_message_as_text(msginfo, prtmp,
+					 conv_get_locale_charset_str(),
+					 all_headers) == 0)
+		print_command_exec(prtmp, cmdline);
 
 	g_free(prtmp);
 }
@@ -1839,7 +1829,7 @@ void procmsg_print_message_part(MsgInfo *msginfo, MimeInfo *partinfo,
 	prtmp = g_strdup_printf("%s%cprinttmp-%08x.txt",
 				get_mime_tmp_dir(), G_DIR_SEPARATOR,
 				print_id++);
-	if ((prfp = g_fopen(prtmp, "wb")) == NULL) {
+	if ((prfp = g_fopen(prtmp, "w")) == NULL) {
 		FILE_OP_ERROR(prtmp, "procmsg_print_message_part: fopen");
 		g_free(prtmp);
 		fclose(tmpfp);
@@ -1857,6 +1847,103 @@ void procmsg_print_message_part(MsgInfo *msginfo, MimeInfo *partinfo,
 	g_free(prtmp);
 }
 
+gint procmsg_save_message_as_text(MsgInfo *msginfo, const gchar *dest,
+				  const gchar *encoding, gboolean all_headers)
+{
+	MimeInfo *mimeinfo, *partinfo;
+	FILE *fp;
+	FILE *tmpfp;
+	FILE *destfp;
+	gchar buf[BUFFSIZE];
+	gchar *part_str;
+	gint ret = 0;
+
+	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(dest != NULL, -1);
+
+	mimeinfo = procmime_scan_message(msginfo);
+	if (!mimeinfo)
+		return -1;
+	if ((fp = procmsg_open_message(msginfo)) == NULL) {
+		procmime_mimeinfo_free_all(mimeinfo);
+		return -1;
+	}
+	if ((destfp = g_fopen(dest, "w")) == NULL) {
+		fclose(fp);
+		procmime_mimeinfo_free_all(mimeinfo);
+		return -1;
+	}
+	procmsg_write_headers(msginfo, mimeinfo, fp, destfp, encoding, all_headers);
+	fputc('\n', destfp);
+
+	partinfo = mimeinfo;
+
+	while (partinfo != NULL) {
+		if (fseek(fp, partinfo->fpos, SEEK_SET) < 0)
+			break;
+
+		if (partinfo->filename || partinfo->name)
+			g_snprintf(buf, sizeof(buf), "\n[%s  %s (%s)]\n",
+				   partinfo->filename ? partinfo->filename :
+				   partinfo->name,
+				   partinfo->content_type,
+				   to_human_readable(partinfo->content_size));
+		else
+			g_snprintf(buf, sizeof(buf), "\n[%s (%s)]\n",
+				   partinfo->content_type,
+				   to_human_readable(partinfo->content_size));
+		part_str = conv_codeset_strdup(buf, CS_INTERNAL, encoding);
+		if (!part_str)
+			part_str = g_strdup(buf);
+
+		if (partinfo->mime_type == MIME_TEXT ||
+		    partinfo->mime_type == MIME_TEXT_HTML) {
+			if (!partinfo->main &&
+			    partinfo->parent &&
+			    partinfo->parent->children != partinfo) {
+				fputs(part_str, destfp);
+			}
+
+			if ((tmpfp = procmime_get_text_content(partinfo, fp, encoding)) == NULL) {
+				g_free(part_str);
+				break;
+			}
+			while (fgets(buf, sizeof(buf), tmpfp) != NULL)
+				fputs(buf, destfp);
+
+			fclose(tmpfp);
+		} else if (partinfo->mime_type == MIME_MESSAGE_RFC822) {
+			fputs(part_str, destfp);
+			while (fgets(buf, sizeof(buf), fp) != NULL)
+				if (buf[0] == '\r' || buf[0] == '\n') break;
+			procmsg_write_headers(msginfo, partinfo, fp, destfp, encoding, all_headers);
+			fputc('\n', destfp);
+		} else if (partinfo->mime_type != MIME_MULTIPART) {
+			fputs(part_str, destfp);
+		}
+
+		g_free(part_str);
+
+		if (partinfo->parent && partinfo->parent->content_type &&
+		    !g_ascii_strcasecmp(partinfo->parent->content_type,
+					"multipart/alternative"))
+			partinfo = partinfo->parent->next;
+		else
+			partinfo = procmime_mimeinfo_next(partinfo);
+	}
+
+	if (fclose(destfp) == EOF) {
+		FILE_OP_ERROR(dest, "fclose");
+		g_unlink(dest);
+		ret = -1;
+	}
+
+	fclose(fp);
+	procmime_mimeinfo_free_all(mimeinfo);
+
+	return ret;
+}
+
 /**
  * procmsg_concat_partial_messages:
  * @mlist: list of MsgInfo* including message/partial messages.
