@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,6 +56,8 @@
 #  include "ssl.h"
 #endif
 
+#include "utils.h"
+
 #define BUFFSIZE	8192
 
 #ifdef G_OS_WIN32
@@ -79,11 +81,18 @@ struct _SockConnectData {
 	gint id;
 	gchar *hostname;
 	gushort port;
+#ifdef G_OS_UNIX
 	GList *addr_list;
 	GList *cur_addr;
 	SockLookupData *lookup_data;
 	GIOChannel *channel;
 	guint io_tag;
+#endif /* G_OS_UNIX */
+#if USE_THREADS
+	gint flag;
+	GThread *thread;
+#endif /* G_OS_UNIX */
+	SockInfo *sock;
 	SockConnectFunc func;
 	gpointer data;
 };
@@ -137,12 +146,26 @@ static gint sock_connect_with_timeout	(gint			 sock,
 					 guint			 timeout_secs);
 
 #ifndef INET6
-static gint sock_connect_by_hostname	(gint		 sock,
-					 const gchar	*hostname,
-					 gushort	 port);
+static gint sock_info_connect_by_hostname
+					(SockInfo	*sock);
 #else
-static SockDesc sock_connect_by_getaddrinfo	(const gchar	*hostname,
-						 gushort	 port);
+#ifdef G_OS_WIN32
+typedef int (*GetAddrInfoFunc)		(const char	*node,
+					 const char	*service,
+					 const struct addrinfo *hints,
+					 struct addrinfo **res);
+typedef void (*FreeAddrInfoFunc)	(struct addrinfo *res);
+
+static GetAddrInfoFunc getaddrinfo_func = NULL;
+static FreeAddrInfoFunc freeaddrinfo_func = NULL;
+
+#undef getaddrinfo
+#define getaddrinfo	my_getaddrinfo
+#undef freeaddrinfo
+#define freeaddrinfo	my_freeaddrinfo
+#endif
+
+static SockDesc sock_info_connect_by_getaddrinfo(SockInfo	*sock);
 #endif
 
 #ifdef G_OS_UNIX
@@ -245,7 +268,8 @@ gint fd_open_inet(gushort port)
 	}
 
 	val = 1;
-	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val)) < 0) {
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&val,
+		       sizeof(val)) < 0) {
 		perror("setsockopt");
 		fd_close(sock);
 		return -1;
@@ -351,6 +375,22 @@ gint fd_accept(gint sock)
 }
 
 
+SockInfo *sock_new(const gchar *hostname, gushort port)
+{
+	SockInfo *sockinfo;
+
+	sockinfo = g_new0(SockInfo, 1);
+	sockinfo->sock = INVALID_SOCKET;
+	sockinfo->sock_ch = NULL;
+	sockinfo->hostname = g_strdup(hostname);
+	sockinfo->port = port;
+	sockinfo->state = CONN_READY;
+	sockinfo->flags = 0;
+	sockinfo->data = NULL;
+
+	return sockinfo;
+}
+
 static SockInfo *sock_find_from_fd(gint fd)
 {
 	GList *cur;
@@ -378,8 +418,13 @@ static gint set_nonblocking_mode(gint fd, gboolean nonblock)
 	}
 
 	sock = sock_find_from_fd(fd);
-	if (sock)
-		sock->nonblock = nonblock;
+	if (sock) {
+		if (nonblock) {
+			SOCK_SET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
+		} else {
+			SOCK_UNSET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
+		}
+	}
 	debug_print("set nonblocking mode to %d\n", nonblock);
 
 	return 0;
@@ -408,8 +453,13 @@ gint sock_set_nonblocking_mode(SockInfo *sock, gboolean nonblock)
 	g_return_val_if_fail(sock != NULL, -1);
 
 	ret = set_nonblocking_mode(sock->sock, nonblock);
-	if (ret == 0)
-		sock->nonblock = nonblock;
+	if (ret == 0) {
+		if (nonblock) {
+			SOCK_SET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
+		} else {
+			SOCK_UNSET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
+		}
+	}
 
 	return ret;
 }
@@ -420,8 +470,9 @@ static gboolean is_nonblocking_mode(gint fd)
 	SockInfo *sock;
 
 	sock = sock_find_from_fd(fd);
-	if (sock)
-		return sock->nonblock;
+	if (sock) {
+		return SOCK_IS_NONBLOCK(sock->flags);
+	}
 
 	return FALSE;
 #else
@@ -442,7 +493,7 @@ gboolean sock_is_nonblocking_mode(SockInfo *sock)
 	g_return_val_if_fail(sock != NULL, FALSE);
 
 #ifdef G_OS_WIN32
-	return sock->nonblock;
+	return SOCK_IS_NONBLOCK(sock->flags);
 #else
 	return is_nonblocking_mode(sock->sock);
 #endif
@@ -554,12 +605,31 @@ guint sock_add_watch(SockInfo *sock, GIOCondition condition, SockFunc func,
 	return g_io_add_watch(sock->sock_ch, condition, sock_watch_cb, sock);
 }
 
+guint sock_add_watch_poll(SockInfo *sock, GIOCondition condition, SockFunc func,
+			  gpointer data)
+{
+	GSource *source;
+
+	sock->callback = func;
+	sock->condition = condition;
+	sock->data = data;
+
+	source = g_source_new(&sock_watch_funcs, sizeof(SockSource));
+	((SockSource *)source)->sock = sock;
+	g_source_set_priority(source, G_PRIORITY_DEFAULT);
+	g_source_set_can_recurse(source, FALSE);
+
+	return g_source_attach(source, NULL);
+}
+
 static gint fd_check_io(gint fd, GIOCondition cond)
 {
 	struct timeval timeout;
 	fd_set fds;
+	SockInfo *sock;
 
-	if (is_nonblocking_mode(fd))
+	sock = sock_find_from_fd(fd);
+	if (sock && !SOCK_IS_CHECK_IO(sock->flags))
 		return 0;
 
 	timeout.tv_sec  = io_timeout;
@@ -584,7 +654,7 @@ static gint fd_check_io(gint fd, GIOCondition cond)
 	}
 }
 
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 static sigjmp_buf jmpenv;
 
 static void timeout_handler(gint sig)
@@ -599,25 +669,120 @@ static gint sock_connect_with_timeout(gint sock,
 				      guint timeout_secs)
 {
 	gint ret;
-#ifdef G_OS_UNIX
-	void (*prev_handler)(gint);
 
-	alarm(0);
-	prev_handler = signal(SIGALRM, timeout_handler);
-	if (sigsetjmp(jmpenv, 1)) {
-		alarm(0);
-		signal(SIGALRM, prev_handler);
-		errno = ETIMEDOUT;
+#ifdef G_OS_WIN32
+	WSAEVENT hevent;
+	gint err;
+	DWORD dwret;
+	WSANETWORKEVENTS events;
+
+	errno = 0;
+
+	hevent = WSACreateEvent();
+	if (hevent == WSA_INVALID_EVENT)
+		return -1;
+
+	ret = WSAEventSelect(sock, hevent, FD_CONNECT);
+	if (ret == SOCKET_ERROR) {
+		g_warning("sock_connect_with_timeout: WSAEventSelect");
+		WSACloseEvent(hevent);
 		return -1;
 	}
-	alarm(timeout_secs);
-#endif
 
 	ret = connect(sock, serv_addr, addrlen);
 
-#ifdef G_OS_UNIX
-	alarm(0);
-	signal(SIGALRM, prev_handler);
+	if (ret == SOCKET_ERROR) {
+		err = WSAGetLastError();
+		if (err != WSAEWOULDBLOCK) {
+			g_warning("sock_connect_with_timeout: connect (%d)", err);
+			ret = -1;
+			goto end;
+		}
+	}
+
+	dwret = WSAWaitForMultipleEvents(1, &hevent, FALSE, timeout_secs * 1000, FALSE);
+	if (dwret == WSA_WAIT_TIMEOUT) {
+		g_warning("sock_connect_with_timeout: timeout");
+		errno = WSAETIMEDOUT;
+		ret = -1;
+		goto end;
+	} else if (dwret != WSA_WAIT_EVENT_0) {
+		g_warning("sock_connect_with_timeout: WSAWaitForMultipleEvents (%d)", dwret);
+		ret = -1;
+		goto end;
+	}
+
+	ret = WSAEnumNetworkEvents(sock, hevent, &events);
+	if (ret == SOCKET_ERROR) {
+		g_warning("sock_connect_with_timeout: WSAEnumNetworkEvents (%d)", ret);
+		ret = -1;
+		goto end;
+	}
+
+	if ((events.lNetworkEvents & FD_CONNECT) &&
+	     events.iErrorCode[FD_CONNECT_BIT] == 0) {
+		ret = 0;
+		errno = 0;
+	} else
+		ret = -1;
+
+end:
+	WSAEventSelect(sock, NULL, 0);
+	WSACloseEvent(hevent);
+
+	set_nonblocking_mode(sock, FALSE);
+#else
+	set_nonblocking_mode(sock, TRUE);
+
+	ret = connect(sock, serv_addr, addrlen);
+
+	if (ret < 0) {
+		if (EINPROGRESS == errno) {
+			fd_set fds;
+			struct timeval tv;
+
+			tv.tv_sec = timeout_secs;
+			tv.tv_usec = 0;
+			FD_ZERO(&fds);
+			FD_SET(sock, &fds);
+			do {
+				ret = select(sock + 1, NULL, &fds, NULL, &tv);
+			} while (ret < 0 && EINTR == errno);
+			if (ret < 0) {
+				perror("sock_connect_with_timeout: select");
+				return -1;
+			} else if (ret == 0) {
+				debug_print("sock_connect_with_timeout: timeout\n");
+				errno = ETIMEDOUT;
+				return -1;
+			} else {
+				gint val;
+				guint len;
+
+				if (FD_ISSET(sock, &fds)) {
+					ret = 0;
+				} else {
+					debug_print("sock_connect_with_timeout: fd not set\n");
+					return -1;
+				}
+
+				len = sizeof(val);
+				if (getsockopt(sock, SOL_SOCKET, SO_ERROR, &val, &len) < 0) {
+					perror("sock_connect_with_timeout: getsockopt");
+					return -1;
+				}
+				if (val != 0) {
+					debug_print("sock_connect_with_timeout: getsockopt(SOL_SOCKET, SO_ERROR) returned error: %s\n", g_strerror(val));
+					return -1;
+				}
+			}
+		} else {
+			perror("sock_connect_with_timeout: connect");
+			return -1;
+		}
+	}
+
+	set_nonblocking_mode(sock, FALSE);
 #endif
 
 	return ret;
@@ -641,7 +806,7 @@ static void resolver_init(void)
 struct hostent *my_gethostbyname(const gchar *hostname)
 {
 	struct hostent *hp;
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 	void (*prev_handler)(gint);
 
 	alarm(0);
@@ -657,7 +822,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 #endif
 
 	if ((hp = gethostbyname(hostname)) == NULL) {
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 		alarm(0);
 		signal(SIGALRM, prev_handler);
 #endif
@@ -666,7 +831,7 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 		return NULL;
 	}
 
-#ifdef G_OS_UNIX
+#if defined(G_OS_UNIX) && !defined(USE_THREADS)
 	alarm(0);
 	signal(SIGALRM, prev_handler);
 #endif
@@ -674,7 +839,34 @@ struct hostent *my_gethostbyname(const gchar *hostname)
 	return hp;
 }
 
-#ifndef INET6
+static void sock_set_buffer_size(gint sock)
+{
+#ifdef G_OS_WIN32
+	gint val;
+	guint len = sizeof(val);
+
+#define SOCK_BUFFSIZE	32768
+
+	getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, &len);
+	if (val < SOCK_BUFFSIZE) {
+		val = SOCK_BUFFSIZE;
+		setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, len);
+	}
+	getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, &len);
+	if (val < SOCK_BUFFSIZE) {
+		val = SOCK_BUFFSIZE;
+		setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, len);
+	}
+	getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&val, &len);
+	debug_print("SO_SNDBUF = %d\n", val);
+	getsockopt(sock, SOL_SOCKET, SO_RCVBUF, (char *)&val, &len);
+	debug_print("SO_RCVBUF = %d\n", val);
+
+#undef SOCK_BUFFSIZE
+#endif
+}
+
+#if !defined(INET6) || defined(G_OS_WIN32)
 static gint my_inet_aton(const gchar *hostname, struct in_addr *inp)
 {
 #if HAVE_INET_ATON
@@ -694,42 +886,177 @@ static gint my_inet_aton(const gchar *hostname, struct in_addr *inp)
 #endif
 #endif /* HAVE_INET_ATON */
 }
+#endif /* !defined(INET6) || defined(G_OS_WIN32) */
 
-static gint sock_connect_by_hostname(gint sock, const gchar *hostname,
-				     gushort port)
+#ifndef INET6
+static gint sock_info_connect_by_hostname(SockInfo *sock)
 {
 	struct hostent *hp;
 	struct sockaddr_in ad;
+	gint ret;
+
+	g_return_val_if_fail(sock != NULL, -1);
+	g_return_val_if_fail(sock->hostname != NULL && sock->port > 0, -1);
 
 	resolver_init();
 
 	memset(&ad, 0, sizeof(ad));
 	ad.sin_family = AF_INET;
-	ad.sin_port = htons(port);
+	ad.sin_port = htons(sock->port);
 
-	if (!my_inet_aton(hostname, &ad.sin_addr)) {
-		if ((hp = my_gethostbyname(hostname)) == NULL) {
-			fprintf(stderr, "%s: unknown host.\n", hostname);
+	if (!my_inet_aton(sock->hostname, &ad.sin_addr)) {
+		if ((hp = my_gethostbyname(sock->hostname)) == NULL) {
+			fprintf(stderr, "%s: unknown host.\n", sock->hostname);
 			errno = 0;
+			sock->state = CONN_LOOKUPFAILED;
 			return -1;
 		}
 
 		if (hp->h_length != 4 && hp->h_length != 8) {
-			fprintf(stderr, "illegal address length received for host %s\n", hostname);
+			fprintf(stderr, "illegal address length received for host %s\n", sock->hostname);
 			errno = 0;
+			sock->state = CONN_LOOKUPFAILED;
 			return -1;
 		}
 
 		memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);
 	}
 
-	return sock_connect_with_timeout(sock, (struct sockaddr *)&ad,
-					 sizeof(ad), io_timeout);
+	sock->state = CONN_LOOKUPSUCCESS;
+
+	ret = sock_connect_with_timeout(sock->sock, (struct sockaddr *)&ad,
+					sizeof(ad), io_timeout);
+	if (ret < 0)
+		sock->state = CONN_FAILED;
+	else
+		sock->state = CONN_ESTABLISHED;
+
+	return ret;
 }
 
 #else /* INET6 */
 
 #ifdef G_OS_WIN32
+static gboolean win32_ipv6_supported(void)
+{
+	static gboolean ipv6_checked = FALSE;
+	HMODULE hmodule;
+
+	if (ipv6_checked)
+		return getaddrinfo_func != NULL;
+
+	hmodule = GetModuleHandleA("ws2_32");
+	if (hmodule) {
+		getaddrinfo_func =
+			(GetAddrInfoFunc)GetProcAddress(hmodule, "getaddrinfo");
+		freeaddrinfo_func =
+			(FreeAddrInfoFunc)GetProcAddress(hmodule, "freeaddrinfo");
+		if (!getaddrinfo_func || !freeaddrinfo_func) {
+			getaddrinfo_func = NULL;
+			freeaddrinfo_func = NULL;
+		}
+	}
+
+	if (getaddrinfo_func)
+		debug_print("ws2_32 has IPv6 functions.\n");
+	else
+		debug_print("ws2_32 does not have IPv6 functions.\n");
+
+	ipv6_checked = TRUE;
+	return getaddrinfo_func != NULL;
+}
+
+/* subset of getaddrinfo() */
+static int my_getaddrinfo(const char *node, const char *service,
+			  const struct addrinfo *hintp,
+			  struct addrinfo **res)
+{
+	struct addrinfo *ai;
+	struct sockaddr_in addr, *paddr;
+	struct addrinfo hints;
+	gint port = 0;
+
+	if (win32_ipv6_supported())
+		return getaddrinfo_func(node, service, hintp, res);
+
+	if (!hintp) {
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = AF_INET;
+		hints.ai_socktype = SOCK_STREAM;
+	} else
+		memcpy(&hints, hintp, sizeof(hints));
+
+	if (hints.ai_family != AF_UNSPEC && hints.ai_family != AF_INET)
+		return EAI_FAMILY;
+	if (hints.ai_socktype == 0)
+		hints.ai_socktype = SOCK_STREAM;
+	if (hints.ai_protocol == 0)
+		hints.ai_protocol = IPPROTO_TCP;
+	if (hints.ai_socktype != SOCK_STREAM)
+		return EAI_SOCKTYPE;
+	if (hints.ai_protocol != IPPROTO_TCP)
+		return EAI_SOCKTYPE;
+#if 0
+	if (!node && !service)
+		return EAI_NONAME;
+#endif
+	if (!node || !service)
+		return EAI_NONAME;
+
+	port = atoi(service);
+
+	memset(&addr, 0, sizeof(addr));
+	addr.sin_family = AF_INET;
+	addr.sin_port = htons(port);
+
+	if (!my_inet_aton(node, &addr.sin_addr)) {
+		struct hostent *hp;
+
+		if ((hp = my_gethostbyname(node)) == NULL) {
+			fprintf(stderr, "%s: unknown host.\n", node);
+			errno = 0;
+			return EAI_NONAME;
+		}
+		if (hp->h_length != 4 && hp->h_length != 8) {
+			fprintf(stderr, "illegal address length received for host %s\n", node);
+			errno = 0;
+			return EAI_FAIL;
+		}
+
+		memcpy(&addr.sin_addr, hp->h_addr, hp->h_length);
+	}
+
+	ai = g_malloc0(sizeof(struct addrinfo));
+	paddr = g_malloc(sizeof(struct sockaddr_in));
+	memcpy(paddr, &addr, sizeof(struct sockaddr_in));
+
+	ai->ai_flags = 0;
+	ai->ai_family = AF_INET;
+	ai->ai_socktype = hints.ai_socktype;
+	ai->ai_protocol = hints.ai_protocol;
+	ai->ai_addrlen = sizeof(struct sockaddr_in);
+	ai->ai_addr = (struct sockaddr *)paddr;
+	ai->ai_canonname = NULL;
+	ai->ai_next = NULL;
+
+	*res = ai;
+
+	return 0;
+}
+
+static void my_freeaddrinfo(struct addrinfo *res)
+{
+	if (win32_ipv6_supported()) {
+		freeaddrinfo_func(res);
+		return;
+	}
+
+	if (res) {
+		g_free(res->ai_addr);
+		g_free(res);
+	}
+}
+
 /* MinGW defines gai_strerror() in ws2tcpip.h, but it is not implemented. */
 #undef gai_strerror
 const gchar *gai_strerror(gint errcode)
@@ -741,13 +1068,16 @@ const gchar *gai_strerror(gint errcode)
 }
 #endif
 
-static SockDesc sock_connect_by_getaddrinfo(const gchar *hostname, gushort port)
+static SockDesc sock_info_connect_by_getaddrinfo(SockInfo *sockinfo)
 {
 	SockDesc sock = INVALID_SOCKET;
 	gint gai_error;
 	struct addrinfo hints, *res, *ai;
 	gchar port_str[6];
 
+	g_return_val_if_fail(sockinfo != NULL, INVALID_SOCKET);
+	g_return_val_if_fail(sockinfo->hostname != NULL && sockinfo->port > 0, INVALID_SOCKET);
+
 	resolver_init();
 
 	memset(&hints, 0, sizeof(hints));
@@ -757,11 +1087,12 @@ static SockDesc sock_connect_by_getaddrinfo(const gchar *hostname, gushort port)
 	hints.ai_protocol = IPPROTO_TCP;
 
 	/* convert port from integer to string. */
-	g_snprintf(port_str, sizeof(port_str), "%d", port);
+	g_snprintf(port_str, sizeof(port_str), "%d", sockinfo->port);
 
-	if ((gai_error = getaddrinfo(hostname, port_str, &hints, &res)) != 0) {
+	if ((gai_error = getaddrinfo(sockinfo->hostname, port_str, &hints, &res)) != 0) {
 		fprintf(stderr, "getaddrinfo for %s:%s failed: %s\n",
-			hostname, port_str, gai_strerror(gai_error));
+			sockinfo->hostname, port_str, gai_strerror(gai_error));
+		sockinfo->state = CONN_LOOKUPFAILED;
 		return INVALID_SOCKET;
 	}
 
@@ -769,6 +1100,7 @@ static SockDesc sock_connect_by_getaddrinfo(const gchar *hostname, gushort port)
 		sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
 		if (!SOCKET_IS_VALID(sock))
 			continue;
+		sock_set_buffer_size(sock);
 
 		if (sock_connect_with_timeout
 			(sock, ai->ai_addr, ai->ai_addrlen, io_timeout) == 0)
@@ -780,22 +1112,45 @@ static SockDesc sock_connect_by_getaddrinfo(const gchar *hostname, gushort port)
 	if (res != NULL)
 		freeaddrinfo(res);
 
-	if (ai == NULL)
+	if (ai == NULL) {
+		sockinfo->state = CONN_FAILED;
 		return INVALID_SOCKET;
+	}
 
+	sockinfo->state = CONN_ESTABLISHED;
 	return sock;
 }
 #endif /* !INET6 */
 
 SockInfo *sock_connect(const gchar *hostname, gushort port)
 {
-	SockDesc sock;
 	SockInfo *sockinfo;
 
-#ifdef INET6
-	sock = sock_connect_by_getaddrinfo(hostname, port);
-	if (!SOCKET_IS_VALID(sock))
+	sockinfo = sock_new(hostname, port);
+	if (sock_info_connect(sockinfo) < 0) {
+		sock_close(sockinfo);
 		return NULL;
+	}
+
+	return sockinfo;
+}
+
+gint sock_info_connect(SockInfo *sockinfo)
+{
+	SockDesc sock;
+#ifndef INET6
+	gint ret;
+#endif
+
+	g_return_val_if_fail(sockinfo != NULL, -1);
+	g_return_val_if_fail(sockinfo->hostname != NULL && sockinfo->port > 0,
+			     -1);
+
+#ifdef INET6
+	sock = sock_info_connect_by_getaddrinfo(sockinfo);
+	if (!SOCKET_IS_VALID(sock)) {
+		return -1;
+	}
 #else
 	sock = socket(AF_INET, SOCK_STREAM, 0);
 	if (!SOCKET_IS_VALID(sock)) {
@@ -804,29 +1159,29 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 #else
 		perror("socket");
 #endif /* G_OS_WIN32 */
-		return NULL;
+		sockinfo->state = CONN_FAILED;
+		return -1;
 	}
+	sock_set_buffer_size(sock);
 
-	if (sock_connect_by_hostname(sock, hostname, port) < 0) {
+	sockinfo->sock = sock;
+	if ((ret = sock_info_connect_by_hostname(sockinfo)) < 0) {
 		if (errno != 0) perror("connect");
 		fd_close(sock);
-		return NULL;
+		sockinfo->sock = INVALID_SOCKET;
+		return ret;
 	}
 #endif /* INET6 */
 
-	sockinfo = g_new0(SockInfo, 1);
 	sockinfo->sock = sock;
 	sockinfo->sock_ch = g_io_channel_unix_new(sock);
-	sockinfo->hostname = g_strdup(hostname);
-	sockinfo->port = port;
-	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->nonblock = FALSE;
+	sockinfo->flags = SYL_SOCK_CHECK_IO;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
 	g_usleep(100000);
 
-	return sockinfo;
+	return 0;
 }
 
 #ifdef G_OS_UNIX
@@ -860,6 +1215,14 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	conn_data->channel = NULL;
 	g_io_channel_unref(source);
 
+	if (condition & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)) {
+		debug_print("sock_connect_async_cb: condition = %d\n",
+			    condition);
+		fd_close(fd);
+		sock_connect_address_list_async(conn_data);
+		return FALSE;
+	}
+
 	len = sizeof(val);
 	if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &val, &len) < 0) {
 		perror("getsockopt");
@@ -869,23 +1232,23 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	}
 
 	if (val != 0) {
+		debug_print("getsockopt(SOL_SOCKET, SO_ERROR) returned error\n");
 		fd_close(fd);
 		sock_connect_address_list_async(conn_data);
 		return FALSE;
 	}
 
-	sockinfo = g_new0(SockInfo, 1);
+	sockinfo = conn_data->sock;
 	sockinfo->sock = fd;
 	sockinfo->sock_ch = g_io_channel_unix_new(fd);
-	sockinfo->hostname = g_strdup(conn_data->hostname);
-	sockinfo->port = conn_data->port;
 	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->nonblock = TRUE;
+	sockinfo->flags = SYL_SOCK_NONBLOCK;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
 	conn_data->func(sockinfo, conn_data->data);
 
+	conn_data->sock = NULL;
 	sock_connect_async_cancel(conn_data->id);
 
 	return FALSE;
@@ -905,23 +1268,41 @@ static gint sock_connect_async_get_address_info_cb(GList *addr_list,
 
 gint sock_connect_async(const gchar *hostname, gushort port,
 			SockConnectFunc func, gpointer data)
+{
+	SockInfo *sock;
+	gint ret;
+
+	sock = sock_new(hostname, port);
+	ret = sock_info_connect_async(sock, func, data);
+	if (ret < 0)
+		sock_close(sock);
+
+	return ret;
+}
+
+gint sock_info_connect_async(SockInfo *sock, SockConnectFunc func,
+			     gpointer data)
 {
 	static gint id = 1;
 	SockConnectData *conn_data;
 
+	g_return_val_if_fail(sock != NULL, -1);
+	g_return_val_if_fail(sock->hostname != NULL && sock->port > 0, -1);
+
 	conn_data = g_new0(SockConnectData, 1);
 	conn_data->id = id++;
-	conn_data->hostname = g_strdup(hostname);
-	conn_data->port = port;
+	conn_data->hostname = g_strdup(sock->hostname);
+	conn_data->port = sock->port;
 	conn_data->addr_list = NULL;
 	conn_data->cur_addr = NULL;
 	conn_data->io_tag = 0;
+	conn_data->sock = sock;
 	conn_data->func = func;
 	conn_data->data = data;
 
 	conn_data->lookup_data = sock_get_address_info_async
-		(hostname, port, sock_connect_async_get_address_info_cb,
-		 conn_data);
+		(sock->hostname, sock->port,
+		 sock_connect_async_get_address_info_cb, conn_data);
 
 	if (conn_data->lookup_data == NULL) {
 		g_free(conn_data->hostname);
@@ -961,6 +1342,8 @@ gint sock_connect_async_cancel(gint id)
 			g_io_channel_shutdown(conn_data->channel, FALSE, NULL);
 			g_io_channel_unref(conn_data->channel);
 		}
+		if (conn_data->sock)
+			sock_close(conn_data->sock);
 
 		sock_address_list_free(conn_data->addr_list);
 		g_free(conn_data->hostname);
@@ -978,6 +1361,15 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 	SockAddrData *addr_data;
 	gint sock = -1;
 
+	if (conn_data->addr_list == NULL) {
+		g_warning("sock_connect_address_list_async: "
+			  "DNS lookup for %s failed", conn_data->hostname);
+		conn_data->sock->state = CONN_LOOKUPFAILED;
+		conn_data->func(conn_data->sock, conn_data->data);
+		sock_connect_async_cancel(conn_data->id);
+		return -1;
+	}
+
 	for (; conn_data->cur_addr != NULL;
 	     conn_data->cur_addr = conn_data->cur_addr->next) {
 		addr_data = (SockAddrData *)conn_data->cur_addr->data;
@@ -988,6 +1380,7 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 			continue;
 		}
 
+		sock_set_buffer_size(sock);
 		set_nonblocking_mode(sock, TRUE);
 
 		if (connect(sock, addr_data->addr, addr_data->addr_len) < 0) {
@@ -1003,18 +1396,22 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 
 	if (conn_data->cur_addr == NULL) {
 		g_warning("sock_connect_address_list_async: "
-			  "connection to %s:%d failed\n",
+			  "connection to %s:%d failed",
 			  conn_data->hostname, conn_data->port);
-		conn_data->func(NULL, conn_data->data);
+		conn_data->sock->state = CONN_FAILED;
+		conn_data->func(conn_data->sock, conn_data->data);
 		sock_connect_async_cancel(conn_data->id);
 		return -1;
 	}
 
+	debug_print("sock_connect_address_list_async: waiting for connect\n");
+
 	conn_data->cur_addr = conn_data->cur_addr->next;
 
 	conn_data->channel = g_io_channel_unix_new(sock);
-	conn_data->io_tag = g_io_add_watch(conn_data->channel, G_IO_OUT,
-					   sock_connect_async_cb, conn_data);
+	conn_data->io_tag = g_io_add_watch
+		(conn_data->channel, G_IO_OUT | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+		 sock_connect_async_cb, conn_data);
 
 	return 0;
 }
@@ -1027,9 +1424,10 @@ static gint sock_kill_process(pid_t pid)
 
 	while (ret == (pid_t)-1) {
 		if ((ret = waitpid(pid, NULL, 0)) != pid) {
-			perror("sock_kill_process(): waitpid");
-			if (ret == (pid_t)-1 && errno != EINTR)
+			if (ret == (pid_t)-1 && errno != EINTR) {
+				perror("sock_kill_process(): waitpid");
 				break;
+			}
 		}
 	}
 
@@ -1062,7 +1460,7 @@ static gboolean sock_get_address_info_async_cb(GIOChannel *source,
 			break;
 
 		if (ai_member[0] == AF_UNSPEC) {
-			g_warning("DNS lookup failed\n");
+			g_warning("DNS lookup failed");
 			break;
 		}
 
@@ -1155,7 +1553,7 @@ static SockLookupData *sock_get_address_info_async(const gchar *hostname,
 
 		gai_err = getaddrinfo(hostname, port_str, &hints, &res);
 		if (gai_err != 0) {
-			g_warning("getaddrinfo for %s:%s failed: %s\n",
+			g_warning("getaddrinfo for %s:%s failed: %s",
 				  hostname, port_str, gai_strerror(gai_err));
 			fd_write_all(pipe_fds[1], (gchar *)ai_member,
 				     sizeof(ai_member));
@@ -1244,6 +1642,123 @@ static gint sock_get_address_info_async_cancel(SockLookupData *lookup_data)
 }
 #endif /* G_OS_UNIX */
 
+#if USE_THREADS
+static gpointer sock_connect_async_func(gpointer data)
+{
+	SockConnectData *conn_data = (SockConnectData *)data;
+	gint ret;
+
+	ret = sock_info_connect(conn_data->sock);
+
+	if (ret == 0) {
+		debug_print("sock_connect_async_func: connected\n");
+	} else {
+		if (conn_data->sock->state == CONN_LOOKUPFAILED)
+			debug_print("sock_connect_async_func: DNS lookup failed\n");
+		else
+			debug_print("sock_connect_async_func: connection failed\n");
+	}
+
+	g_atomic_int_set(&conn_data->flag, 1);
+	g_main_context_wakeup(NULL);
+
+	debug_print("sock_connect_async_func: exit\n");
+	return GINT_TO_POINTER(ret);
+}
+
+gint sock_connect_async_thread(const gchar *hostname, gushort port)
+{
+	SockInfo *sock;
+	gint ret;
+
+	sock = sock_new(hostname, port);
+	ret = sock_info_connect_async_thread(sock);
+	if (ret < 0)
+		sock_close(sock);
+
+	return ret;
+}
+
+gint sock_info_connect_async_thread(SockInfo *sock)
+{
+	static gint id = 1;
+	SockConnectData *data;
+
+	g_return_val_if_fail(sock != NULL, -1);
+	g_return_val_if_fail(sock->hostname != NULL && sock->port > 0, -1);
+
+	data = g_new0(SockConnectData, 1);
+	data->id = id++;
+	data->hostname = g_strdup(sock->hostname);
+	data->port = sock->port;
+	data->flag = 0;
+	data->sock = sock;
+
+	data->thread = g_thread_create(sock_connect_async_func, data, TRUE,
+				       NULL);
+	if (!data->thread) {
+		g_free(data->hostname);
+		g_free(data);
+		return -1;
+	}
+
+	sock_connect_data_list = g_list_append(sock_connect_data_list, data);
+
+	return data->id;
+}
+
+gint sock_connect_async_thread_wait(gint id, SockInfo **sock)
+{
+	gint ret;
+
+	*sock = NULL;
+	ret = sock_info_connect_async_thread_wait(id, sock);
+	if (ret < 0) {
+		if (*sock) {
+			sock_close(*sock);
+			*sock = NULL;
+		}
+	}
+
+	return ret;
+}
+
+gint sock_info_connect_async_thread_wait(gint id, SockInfo **sock)
+{
+	SockConnectData *conn_data = NULL;
+	GList *cur;
+	gint ret;
+
+	for (cur = sock_connect_data_list; cur != NULL; cur = cur->next) {
+		if (((SockConnectData *)cur->data)->id == id) {
+			conn_data = (SockConnectData *)cur->data;
+			break;
+		}
+	}
+
+	if (!conn_data) {
+		g_warning("sock_info_connect_async_thread_wait: id %d not found.", id);
+		return -1;
+	}
+
+	debug_print("sock_connect_async_thread_wait: waiting thread\n");
+	while (g_atomic_int_get(&conn_data->flag) == 0)
+		event_loop_iterate();
+
+	ret = GPOINTER_TO_INT(g_thread_join(conn_data->thread));
+	debug_print("sock_info_connect_async_thread_wait: thread exited with status %d\n", ret);
+
+	if (sock)
+		*sock = conn_data->sock;
+
+	sock_connect_data_list = g_list_remove(sock_connect_data_list,
+					       conn_data);
+	g_free(conn_data->hostname);
+	g_free(conn_data);
+
+	return ret;
+}
+#endif /* USE_THREADS */
 
 gint sock_printf(SockInfo *sock, const gchar *format, ...)
 {
@@ -1300,6 +1815,8 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 {
 	gint err, ret;
 
+	errno = 0;
+
 	if (SSL_pending(ssl) == 0) {
 		if (fd_check_io(SSL_get_rfd(ssl), G_IO_IN) < 0)
 			return -1;
@@ -1320,6 +1837,9 @@ gint ssl_read(SSL *ssl, gchar *buf, gint len)
 		g_warning("SSL_read() returned error %d, ret = %d\n", err, ret);
 		if (ret == 0)
 			return 0;
+#ifdef G_OS_WIN32
+		errno = EIO;
+#endif
 		return -1;
 	}
 }
@@ -1628,6 +2148,8 @@ gint sock_close(SockInfo *sock)
 	if (!sock)
 		return 0;
 
+	debug_print("sock_close: %s:%u (%p)\n", sock->hostname ? sock->hostname : "(none)", sock->port, sock);
+
 #if USE_SSL
 	if (sock->ssl)
 		ssl_done_socket(sock);
