@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2010 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -52,6 +52,10 @@
 #  include <shlobj.h>
 #endif
 
+#ifdef __APPLE__
+#  include <AppKit/AppKit.h>
+#endif
+
 #include "utils.h"
 #include "socket.h"
 
@@ -354,20 +358,25 @@ gchar *utos_buf(gchar *nstr, guint n)
 	return nstr;
 }
 
+gchar *to_human_readable_buf(gchar *buf, size_t bufsize, gint64 size)
+{
+	if (size < 1024)
+		g_snprintf(buf, bufsize, "%dB", (gint)size);
+	else if ((size >> 10) < 1024)
+		g_snprintf(buf, bufsize, "%.1fKB", (gfloat)size / (1 << 10));
+	else if ((size >> 20) < 1024)
+		g_snprintf(buf, bufsize, "%.2fMB", (gfloat)size / (1 << 20));
+	else
+		g_snprintf(buf, bufsize, "%.2fGB", (gfloat)size / (1 << 30));
+
+	return buf;
+}
+
 gchar *to_human_readable(gint64 size)
 {
 	static gchar str[16];
 
-	if (size < 1024)
-		g_snprintf(str, sizeof(str), "%dB", (gint)size);
-	else if ((size >> 10) < 1024)
-		g_snprintf(str, sizeof(str), "%.1fKB", (gfloat)size / (1 << 10));
-	else if ((size >> 20) < 1024)
-		g_snprintf(str, sizeof(str), "%.2fMB", (gfloat)size / (1 << 20));
-	else
-		g_snprintf(str, sizeof(str), "%.2fGB", (gfloat)size / (1 << 30));
-
-	return str;
+	return to_human_readable_buf(str, sizeof(str), size);
 }
 
 /* strcmp with NULL-checking */
@@ -2124,6 +2133,21 @@ void set_startup_dir(void)
 		} else
 			startup_dir = g_get_current_dir();
 	}
+#elif defined(__APPLE__)
+	if (!startup_dir) {
+		const gchar *path;
+		NSAutoreleasePool *pool;
+
+		pool = [[NSAutoreleasePool alloc] init];
+
+		path = [[[NSBundle mainBundle] bundlePath] UTF8String];
+		startup_dir = g_strdup(path);
+
+		[pool release];
+
+		if (!startup_dir)
+			startup_dir = g_get_current_dir();
+	}
 #else
 	if (!startup_dir)
 		startup_dir = g_get_current_dir();
@@ -3118,6 +3142,35 @@ gint copy_file_part(FILE *fp, off_t offset, size_t length, const gchar *dest)
 	return 0;
 }
 
+gint copy_file_stream(FILE *fp, FILE *dest_fp)
+{
+	gint n_read;
+	gchar buf[BUFFSIZE];
+
+	g_return_val_if_fail(fp != NULL, -1);
+	g_return_val_if_fail(dest_fp != NULL, -1);
+
+	while ((n_read = fread(buf, sizeof(gchar), sizeof(buf), fp)) > 0) {
+		if (n_read < sizeof(buf) && ferror(fp))
+			break;
+		if (fwrite(buf, n_read, 1, dest_fp) < 1) {
+			g_warning("copy_file_stream: writing to file failed.\n");
+			return -1;
+		}
+	}
+
+	if (ferror(fp)) {
+		perror("fread");
+		return -1;
+	}
+	if (fflush(dest_fp) == EOF) {
+		FILE_OP_ERROR("copy_file_stream", "fflush");
+		return -1;
+	}
+
+	return 0;
+}
+
 /* convert line endings into CRLF. If the last line doesn't end with
  * linebreak, add it.
  */
@@ -4243,6 +4296,25 @@ gint open_uri(const gchar *uri, const gchar *cmdline)
 	return 0;
 }
 
+gint play_sound(const gchar *file, gboolean async)
+{
+#ifdef G_OS_WIN32
+	wchar_t *wfile;
+	DWORD flag = SND_FILENAME;
+
+	wfile = g_utf8_to_utf16(file, -1, NULL, NULL, NULL);
+	if (wfile == NULL)
+		return -1;
+	if (async)
+		flag |= SND_ASYNC;
+	else
+		flag |= SND_SYNC;
+	PlaySoundW(wfile, NULL, flag);
+	g_free(wfile);
+#endif
+	return 0;
+}
+
 time_t remote_tzoffset_sec(const gchar *zone)
 {
 	static gchar ustzstr[] = "PSTPDTMSTMDTCSTCDTESTEDT";
