@@ -264,7 +264,7 @@ void addrindex_free_datasource( AddressIndex *addrIndex, AddressDataSource *ds )
 	g_return_if_fail( addrIndex != NULL );
 	g_return_if_fail( ds != NULL );
 
-	if( ds->interface == NULL ) {
+	if( ds->iface == NULL ) {
 		iface = addrindex_get_interface( addrIndex, ds->type );
 	}
 	if( iface == NULL ) return;
@@ -314,11 +314,11 @@ void addrindex_free_datasource( AddressIndex *addrIndex, AddressDataSource *ds )
 	ADDRITEM_SUBTYPE(addrIndex) = 0;
 	ds->type = ADDR_IF_NONE;
 	ds->rawDataSource = NULL;
-	ds->interface = NULL;
+	ds->iface = NULL;
 
 	ds->type = ADDR_IF_NONE;
 	ds->rawDataSource = NULL;
-	ds->interface = NULL;
+	ds->iface = NULL;
 	g_free( ds );
 }
 
@@ -363,7 +363,7 @@ static void addrindex_free_all_datasources( AddressInterface *iface ) {
 
 		ds->type = ADDR_IF_NONE;
 		ds->rawDataSource = NULL;
-		ds->interface = NULL;
+		ds->iface = NULL;
 		g_free( ds );
 		node->data = NULL;
 		node = g_list_next( node );
@@ -525,7 +525,7 @@ AddressDataSource *addrindex_create_datasource() {
 	ADDRITEM_SUBTYPE(ds) = 0;
 	ds->type = ADDR_IF_NONE;
 	ds->rawDataSource = NULL;
-	ds->interface = NULL;
+	ds->iface = NULL;
 	return ds;
 }
 
@@ -551,7 +551,7 @@ AddressDataSource *addrindex_index_add_datasource( AddressIndex *addrIndex, Addr
 		ADDRITEM_PARENT(ds) = ADDRITEM_OBJECT(iface);
 		ds->type = ifType;
 		ds->rawDataSource = dataSource;
-		ds->interface = iface;
+		ds->iface = iface;
 		iface->listSource = g_list_append( iface->listSource, ds );
 		addrIndex->dirtyFlag = TRUE;
 	}
@@ -576,7 +576,7 @@ AddressDataSource *addrindex_index_remove_datasource( AddressIndex *addrIndex, A
 	if( iface ) {
 		iface->listSource = g_list_remove( iface->listSource, dataSource );
 		addrIndex->dirtyFlag = TRUE;
-		dataSource->interface = NULL;
+		dataSource->iface = NULL;
 		retVal = dataSource;
 	}
 	return retVal;
@@ -1003,7 +1003,7 @@ static void addrindex_read_index( AddressIndex *addrIndex, XMLFile *file ) {
 				}
 				if( ds ) {
 					ds->type = addrIndex->lastType;
-					ds->interface = dsIFace;
+					ds->iface = dsIFace;
 					dsIFace->listSource = g_list_append( dsIFace->listSource, ds );
 				}
 				/* printf( "=============================\n\n" ); */
@@ -1637,7 +1637,7 @@ static gboolean addrindex_create_new_book( AddressIndex *addrIndex, gchar *displ
 * addrIndex. Three files will be created, for the following:
 *	"Common addresses"
 *	"Personal addresses"
-*	"Gathered addresses" - a new address book.
+*	"Auto-registered" - a new address book.
 */
 gint addrindex_read_data( AddressIndex *addrIndex ) {
 	g_return_val_if_fail( addrIndex != NULL, -1 );
@@ -1666,7 +1666,7 @@ gint addrindex_read_data( AddressIndex *addrIndex ) {
 * addrIndex. Three files will be created, for the following:
 *	"Common addresses"
 *	"Personal addresses"
-*	"Gathered addresses" - a new address book.
+*	"Auto-registered" - a new address book.
 */
 gint addrindex_create_new_books( AddressIndex *addrIndex ) {
 	gboolean flg;
@@ -1676,11 +1676,44 @@ gint addrindex_create_new_books( AddressIndex *addrIndex ) {
 	flg = addrindex_create_new_book( addrIndex, DISP_NEW_COMMON );
 	if( flg ) {
 		flg = addrindex_create_new_book( addrIndex, DISP_NEW_PERSONAL );
+		flg = addrindex_create_new_book( addrIndex, ADDR_DS_AUTOREG );
 		addrIndex->dirtyFlag = TRUE;
 	}
 	return addrIndex->retVal;
 }
 
+gint addrindex_create_extra_books( AddressIndex *addrIndex ) {
+	GList *node_ds;
+	AddressInterface *iface = NULL;
+	AddressDataSource *ds = NULL;
+	const gchar *ds_name;
+
+	g_return_val_if_fail(addrIndex != NULL, -1);
+
+	iface = addrindex_get_interface(addrIndex, ADDR_IF_BOOK);
+	if (!iface)
+		return -1;
+
+	for (node_ds = iface->listSource; node_ds != NULL;
+	     node_ds = node_ds->next) {
+		ds = node_ds->data;
+		ds_name = addrindex_ds_get_name(ds);
+		if (!ds_name)
+			continue;
+		if (!strcmp(ds_name, ADDR_DS_AUTOREG)) {
+			debug_print("%s found\n", ADDR_DS_AUTOREG);
+			return 0;
+		}
+	}
+
+	debug_print("%s not found, creating new one\n", ADDR_DS_AUTOREG);
+	if (addrindex_create_new_book(addrIndex, ADDR_DS_AUTOREG)) {
+		addrIndex->dirtyFlag = TRUE;
+	}
+
+	return addrIndex->retVal;
+}
+
 /* **********************************************************************
 * New interface stuff.
 * ***********************************************************************
@@ -1694,7 +1727,7 @@ gboolean addrindex_ds_get_modify_flag( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getModifyFlag ) {
 		retVal = ( iface->getModifyFlag ) ( ds->rawDataSource );
@@ -1710,7 +1743,7 @@ gboolean addrindex_ds_get_access_flag( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getAccessFlag ) {
 		retVal = ( iface->getAccessFlag ) ( ds->rawDataSource );
@@ -1726,7 +1759,7 @@ gboolean addrindex_ds_get_read_flag( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getReadFlag ) {
 		retVal = ( iface->getReadFlag ) ( ds->rawDataSource );
@@ -1742,7 +1775,7 @@ gint addrindex_ds_get_status_code( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getStatusCode ) {
 		retVal = ( iface->getStatusCode ) ( ds->rawDataSource );
@@ -1758,7 +1791,7 @@ gint addrindex_ds_read_data( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getReadData ) {
 		retVal = ( iface->getReadData ) ( ds->rawDataSource );
@@ -1774,7 +1807,7 @@ ItemFolder *addrindex_ds_get_root_folder( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getRootFolder ) {
 		retVal = ( iface->getRootFolder ) ( ds->rawDataSource );
@@ -1790,7 +1823,7 @@ GList *addrindex_ds_get_list_folder( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getListFolder ) {
 		retVal = ( iface->getListFolder ) ( ds->rawDataSource );
@@ -1806,7 +1839,7 @@ GList *addrindex_ds_get_list_person( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getListPerson ) {
 		retVal = ( iface->getListPerson ) ( ds->rawDataSource );
@@ -1822,7 +1855,7 @@ gchar *addrindex_ds_get_name( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getName ) {
 		retVal = ( iface->getName ) ( ds->rawDataSource );
@@ -1837,7 +1870,7 @@ void addrindex_ds_set_access_flag( AddressDataSource *ds, gboolean *value ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return;
 	if( iface->setAccessFlag ) {
 		( iface->setAccessFlag ) ( ds->rawDataSource, value );
@@ -1850,7 +1883,7 @@ void addrindex_ds_set_access_flag( AddressDataSource *ds, gboolean *value ) {
 gboolean addrindex_ds_get_readonly( AddressDataSource *ds ) {
 	AddressInterface *iface;
 	if( ds == NULL ) return TRUE;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return TRUE;
 	return iface->readOnly;
 }
@@ -1863,7 +1896,7 @@ GList *addrindex_ds_get_all_persons( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getAllPersons ) {
 		retVal = ( iface->getAllPersons ) ( ds->rawDataSource );
@@ -1879,7 +1912,7 @@ GList *addrindex_ds_get_all_groups( AddressDataSource *ds ) {
 	AddressInterface *iface;
 
 	if( ds == NULL ) return retVal;
-	iface = ds->interface;
+	iface = ds->iface;
 	if( iface == NULL ) return retVal;
 	if( iface->getAllGroups ) {
 		retVal = ( iface->getAllGroups ) ( ds->rawDataSource );
