@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <gtk/gtkitemfactory.h>
 #include <gtk/gtktexttag.h>
 #include <gtk/gtkliststore.h>
+#include <gtk/gtktooltips.h>
 
 typedef struct _Compose		Compose;
 typedef struct _AttachInfo	AttachInfo;
@@ -196,6 +197,10 @@ struct _Compose
 	guint lock_count;
 
 	gboolean window_maximized;
+
+	gboolean block_modified;
+
+	GtkTooltips *toolbar_tip;
 };
 
 struct _AttachInfo
@@ -238,6 +243,9 @@ gchar *compose_entry_get_text	(Compose	  *compose,
 void compose_lock		(Compose	*compose);
 void compose_unlock		(Compose	*compose);
 
+void compose_block_modified	(Compose	*compose);
+void compose_unblock_modified	(Compose	*compose);
+
 void compose_reflect_prefs_all	(void);
 
 #endif /* __COMPOSE_H__ */
