@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,12 +58,36 @@
 #include "progressdialog.h"
 #include "alertpanel.h"
 #include "trayicon.h"
+#include "notificationwindow.h"
 #include "filter.h"
 #include "folder.h"
 #include "procheader.h"
+#include "plugin.h"
+
+
+typedef struct _IncAccountNewMsgCount
+{
+	PrefsAccount *account;
+	gint new_messages;
+} IncAccountNewMsgCount;
+
+typedef struct _IncMsgSummary
+{
+	gchar *subject;
+	gchar *from;
+} IncMsgSummary;
+
+struct _IncResult
+{
+	GSList *count_list;
+	GSList *msg_summaries;
+};
+
 
 static GList *inc_dialog_list = NULL;
 
+static gboolean inc_is_running = FALSE;
+
 static guint inc_lock_count = 0;
 static gboolean block_notify = FALSE;
 
@@ -71,13 +95,20 @@ static GdkPixbuf *current_pixbuf;
 static GdkPixbuf *error_pixbuf;
 static GdkPixbuf *ok_pixbuf;
 
+
 static void inc_finished		(MainWindow		*mainwin,
+					 IncResult		*result);
+static GSList *inc_add_message_count	(GSList			*list,
+					 PrefsAccount		*account,
 					 gint			 new_messages);
+static void inc_result_free		(IncResult		*result,
+					 gboolean		 free_self);
 
 static gint inc_remote_account_mail	(MainWindow		*mainwin,
 					 PrefsAccount		*account);
 static gint inc_account_mail_real	(MainWindow		*mainwin,
-					 PrefsAccount		*account);
+					 PrefsAccount		*account,
+					 IncResult		*result);
 
 static IncProgressDialog *inc_progress_dialog_create
 					(gboolean		 autocheck);
@@ -122,8 +153,9 @@ static gint inc_recv_message		(Session	*session,
 static gint inc_drop_message		(Pop3Session	*session,
 					 const gchar	*file);
 
-static void inc_put_error		(IncState	 istate,
-					 const gchar	*msg);
+static void inc_put_error		(IncSession	*session,
+					 IncState	 istate,
+					 const gchar	*pop3_msg);
 
 static void inc_cancel_cb		(GtkWidget	*widget,
 					 gpointer	 data);
@@ -140,33 +172,92 @@ static gint get_spool			(FolderItem	*dest,
 static void inc_autocheck_timer_set_interval	(guint		 interval);
 static gint inc_autocheck_func			(gpointer	 data);
 
+
 /**
  * inc_finished:
  * @mainwin: Main window.
+ * @result: Information of incorporation result.
  * @new_messages: Number of received messages.
  * 
  * Update the folder view and the summary view after receiving
  * messages.  If @new_messages is 0, this function avoids unneeded
  * updating.
  **/
-static void inc_finished(MainWindow *mainwin, gint new_messages)
+static void inc_finished(MainWindow *mainwin, IncResult *result)
 {
 	FolderItem *item;
+	gint new_messages = 0;
+	gint other_new = 0;
+	IncAccountNewMsgCount *count;
+	GSList *cur;
 
-	debug_print("inc_finished(): %d new message(s)\n", new_messages);
+	if (result) {
+		for (cur = result->count_list; cur != NULL; cur = cur->next) {
+			count = cur->data;
+			if (count->new_messages > 0)
+				new_messages += count->new_messages;
+		}
+	}
 
-	if (prefs_common.scan_all_after_inc)
-		new_messages += folderview_check_new(NULL);
+	debug_print("inc_finished: %d new message(s)\n", new_messages);
+
+	if (prefs_common.scan_all_after_inc) {
+		other_new = folderview_check_new(NULL);
+		new_messages += other_new;
+	}
 
 	if (new_messages > 0 && !block_notify) {
-		gchar buf[1024];
+		GString *str;
+		gint c = 0;
 
-		g_snprintf(buf, sizeof(buf), _("Sylpheed: %d new messages"),
-			   new_messages);
-		trayicon_set_tooltip(buf);
+		str = g_string_new("");
+		g_string_printf(str, _("Sylpheed: %d new messages"),
+				new_messages);
+		if (result) {
+			for (cur = result->count_list; cur != NULL; cur = cur->next) {
+				count = cur->data;
+				if (count->new_messages > 0) {
+					if (c == 0)
+						g_string_append(str, "\n");
+					c++;
+					g_string_append(str, "\n");
+					if (!count->account)
+						g_string_append_printf(str, _("[Local]: %d"), count->new_messages);
+					else
+						g_string_append_printf(str, "%s: %d", count->account->account_name ? count->account->account_name : "[?]", count->new_messages);
+				}
+			}
+		}
+		debug_print("inc_finished: %s\n", str->str);
+		trayicon_set_tooltip(str->str);
 		trayicon_set_notify(TRUE);
+
+		if (prefs_common.enable_newmsg_notify_window) {
+			gchar buf[1024];
+
+			g_snprintf(buf, sizeof(buf), _("Sylpheed: %d new messages"), new_messages);
+			g_string_truncate(str, 0);
+			if (result) {
+				for (cur = result->msg_summaries; cur != NULL; cur = cur->next) {
+					IncMsgSummary *summary = cur->data;
+					gchar *markup;
+
+					if (str->len > 0)
+						g_string_append_c(str, '\n');
+					markup = g_markup_printf_escaped("<b>%s</b>  %s", summary->subject, summary->from);
+					g_string_append(str, markup);
+					g_free(markup);
+				}
+			}
+
+			notification_window_open(buf, str->str, prefs_common.notify_window_period);
+		}
+
+		g_string_free(str, TRUE);
 	}
 
+	syl_plugin_signal_emit("inc-mail-finished", new_messages);
+
 	inc_block_notify(FALSE);
 
 	if (new_messages <= 0 && !prefs_common.scan_all_after_inc) return;
@@ -182,6 +273,14 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 			folderview_update_item(item, TRUE);
 	}
 
+	/* Notification */
+
+	if (new_messages > 0 &&
+	    prefs_common.enable_newmsg_notify_sound &&
+	    prefs_common.newmsg_notify_sound) {
+		play_sound(prefs_common.newmsg_notify_sound, TRUE);
+	}
+
 	if (new_messages > 0 &&
 	    prefs_common.enable_newmsg_notify &&
 	    prefs_common.newmsg_notify_cmd) {
@@ -199,8 +298,39 @@ static void inc_finished(MainWindow *mainwin, gint new_messages)
 	}
 }
 
+static GSList *inc_add_message_count(GSList *list, PrefsAccount *account,
+				     gint new_messages)
+{
+	IncAccountNewMsgCount *count;
+
+	count = g_new(IncAccountNewMsgCount, 1);
+	count->account = account;
+	count->new_messages = new_messages;
+
+	return g_slist_append(list, count);
+}
+
+static void inc_result_free(IncResult *result, gboolean free_self)
+{
+	GSList *cur;
+
+	slist_free_strings(result->count_list);
+	g_slist_free(result->count_list);
+	for (cur = result->msg_summaries; cur != NULL; cur = cur->next) {
+		IncMsgSummary *sum = cur->data;
+		g_free(sum->subject);
+		g_free(sum->from);
+		g_free(sum);
+	}
+	g_slist_free(result->msg_summaries);
+
+	if (free_self)
+		g_free(result);
+}
+
 void inc_mail(MainWindow *mainwin)
 {
+	IncResult result = {NULL, NULL};
 	gint new_msgs = 0;
 
 	if (inc_lock_count) return;
@@ -209,31 +339,43 @@ void inc_mail(MainWindow *mainwin)
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return;
 
+	inc_is_running = TRUE;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
 	main_window_lock(mainwin);
 
+	syl_plugin_signal_emit("inc-mail-start", cur_account);
+
 	if (prefs_common.use_extinc && prefs_common.extinc_cmd) {
 		/* external incorporating program */
 		if (execute_command_line(prefs_common.extinc_cmd, FALSE) != 0) {
+			inc_is_running = FALSE;
 			main_window_unlock(mainwin);
 			inc_autocheck_timer_set();
 			return;
 		}
 
-		if (prefs_common.inc_local)
+		if (prefs_common.inc_local) {
 			new_msgs = inc_spool();
+			result.count_list = inc_add_message_count(result.count_list, NULL, new_msgs);
+		}
 	} else {
 		if (prefs_common.inc_local) {
 			new_msgs = inc_spool();
 			if (new_msgs < 0)
 				new_msgs = 0;
+			result.count_list = inc_add_message_count(result.count_list, NULL, new_msgs);
 		}
 
-		new_msgs += inc_account_mail_real(mainwin, cur_account);
+		new_msgs = inc_account_mail_real(mainwin, cur_account, &result);
 	}
 
-	inc_finished(mainwin, new_msgs);
+	inc_finished(mainwin, &result);
+	inc_result_free(&result, FALSE);
+
+	inc_is_running = FALSE;
+
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 }
@@ -368,7 +510,8 @@ static gint inc_remote_account_mail(MainWindow *mainwin, PrefsAccount *account)
 	return new_msgs;
 }
 
-static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
+static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account,
+				  IncResult *result)
 {
 	IncProgressDialog *inc_dialog;
 	IncSession *session;
@@ -384,6 +527,7 @@ static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
 	inc_dialog = inc_progress_dialog_create(FALSE);
 	inc_dialog->queue_list = g_list_append(inc_dialog->queue_list, session);
 	inc_dialog->mainwin = mainwin;
+	inc_dialog->result = result;
 	inc_progress_dialog_set_list(inc_dialog);
 
 	main_window_set_toolbar_sensitive(mainwin);
@@ -394,6 +538,7 @@ static gint inc_account_mail_real(MainWindow *mainwin, PrefsAccount *account)
 
 gint inc_account_mail(MainWindow *mainwin, PrefsAccount *account)
 {
+	IncResult result = {NULL, NULL};
 	gint new_msgs;
 
 	if (inc_lock_count) return 0;
@@ -402,13 +547,21 @@ gint inc_account_mail(MainWindow *mainwin, PrefsAccount *account)
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return 0;
 
+	inc_is_running = TRUE;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
 	main_window_lock(mainwin);
 
-	new_msgs = inc_account_mail_real(mainwin, account);
+	syl_plugin_signal_emit("inc-mail-start", account);
+
+	new_msgs = inc_account_mail_real(mainwin, account, &result);
+
+	inc_finished(mainwin, &result);
+	inc_result_free(&result, FALSE);
+
+	inc_is_running = FALSE;
 
-	inc_finished(mainwin, new_msgs);
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 
@@ -419,6 +572,7 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 {
 	GList *list, *queue_list = NULL;
 	IncProgressDialog *inc_dialog;
+	IncResult result = {NULL, NULL};
 	gint new_msgs = 0;
 
 	if (inc_lock_count) return;
@@ -427,22 +581,29 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return;
 
+	inc_is_running = TRUE;
+
 	inc_autocheck_timer_remove();
 	summary_write_cache(mainwin->summaryview);
 	main_window_lock(mainwin);
 
+	syl_plugin_signal_emit("inc-mail-start", NULL);
+
 	if (prefs_common.inc_local) {
 		new_msgs = inc_spool();
 		if (new_msgs < 0)
 			new_msgs = 0;
+		result.count_list = inc_add_message_count(result.count_list, NULL, new_msgs);
 	}
 
 	/* check IMAP4 / News folders */
 	for (list = account_get_list(); list != NULL; list = list->next) {
 		PrefsAccount *account = list->data;
 		if ((account->protocol == A_IMAP4 ||
-		     account->protocol == A_NNTP) && account->recv_at_getall)
-			new_msgs += inc_remote_account_mail(mainwin, account);
+		     account->protocol == A_NNTP) && account->recv_at_getall) {
+			new_msgs = inc_remote_account_mail(mainwin, account);
+			result.count_list = inc_add_message_count(result.count_list, account, new_msgs);
+		}
 	}
 
 	/* check POP3 accounts */
@@ -461,15 +622,20 @@ void inc_all_account_mail(MainWindow *mainwin, gboolean autocheck)
 		inc_dialog = inc_progress_dialog_create(autocheck);
 		inc_dialog->queue_list = queue_list;
 		inc_dialog->mainwin = mainwin;
+		inc_dialog->result = &result;
 		inc_progress_dialog_set_list(inc_dialog);
 
 		main_window_set_toolbar_sensitive(mainwin);
 		main_window_set_menu_sensitive(mainwin);
 
-		new_msgs += inc_start(inc_dialog);
+		inc_start(inc_dialog);
 	}
 
-	inc_finished(mainwin, new_msgs);
+	inc_finished(mainwin, &result);
+	inc_result_free(&result, FALSE);
+
+	inc_is_running = FALSE;
+
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 }
@@ -487,6 +653,8 @@ gint inc_pop_before_smtp(PrefsAccount *account)
 	if (!main_window_toggle_online_if_offline(mainwin))
 		return -1;
 
+	inc_is_running = TRUE;
+
 	inc_autocheck_timer_remove();
 	main_window_lock(mainwin);
 
@@ -499,6 +667,7 @@ gint inc_pop_before_smtp(PrefsAccount *account)
 			     _("Authenticating with POP3"));
 	inc_dialog->queue_list = g_list_append(inc_dialog->queue_list, session);
 	inc_dialog->mainwin = mainwin;
+	inc_dialog->result = NULL;
 	inc_progress_dialog_set_list(inc_dialog);
 	inc_dialog->show_dialog = TRUE;
 
@@ -507,6 +676,8 @@ gint inc_pop_before_smtp(PrefsAccount *account)
 
 	inc_start(inc_dialog);
 
+	inc_is_running = FALSE;
+
 	main_window_unlock(mainwin);
 	inc_autocheck_timer_set();
 
@@ -756,6 +927,10 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			SET_PIXMAP_AND_TEXT(ok_pixbuf, _("Done"), msg);
 			g_free(msg);
 			break;
+		case INC_LOOKUP_ERROR:
+			SET_PIXMAP_AND_TEXT(error_pixbuf,
+					    _("Server not found"), NULL);
+			break;
 		case INC_CONNECT_ERROR:
 			SET_PIXMAP_AND_TEXT(error_pixbuf,
 					    _("Connection failed"), NULL);
@@ -784,6 +959,8 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 			break;
 		}
 
+		if (inc_dialog->result)
+			inc_dialog->result->count_list = inc_add_message_count(inc_dialog->result->count_list, pop3_session->ac_prefs, session->new_msgs);
 		new_msgs += session->new_msgs;
 
 		if (!prefs_common.scan_all_after_inc) {
@@ -804,7 +981,8 @@ static gint inc_start(IncProgressDialog *inc_dialog)
 				manage_window_focus_in
 					(inc_dialog->dialog->window,
 					 NULL, NULL);
-			inc_put_error(inc_state, pop3_session->error_msg);
+			inc_put_error(session, inc_state,
+				      pop3_session->error_msg);
 			if (inc_dialog->show_dialog)
 				manage_window_focus_out
 					(inc_dialog->dialog->window,
@@ -904,8 +1082,10 @@ static IncState inc_pop3_session_do(IncSession *session)
 			    SESSION(pop3_session)->server,
 			    SESSION(pop3_session)->port);
 		session->inc_state = INC_CONNECT_ERROR;
+		if (session_get_error(SESSION(pop3_session)) == SESSION_ERROR_LOOKUP)
+			session->inc_state = INC_LOOKUP_ERROR;
 		statusbar_pop_all();
-		return INC_CONNECT_ERROR;
+		return session->inc_state;
 	}
 
 	while (session_is_connected(SESSION(pop3_session)) &&
@@ -914,6 +1094,10 @@ static IncState inc_pop3_session_do(IncSession *session)
 	}
 	log_window_flush();
 
+	debug_print("inc_state: %d\n", session->inc_state);
+	debug_print("pop3_session.error_val: %d\n", pop3_session->error_val);
+	debug_print("pop3_session.error_msg: %s\n", pop3_session->error_msg ? pop3_session->error_msg : "(empty)");
+
 	if (session->inc_state == INC_SUCCESS) {
 		switch (pop3_session->error_val) {
 		case PS_SUCCESS:
@@ -923,6 +1107,8 @@ static IncState inc_pop3_session_do(IncSession *session)
 					session->inc_state = INC_CONNECT_ERROR;
 				else
 					session->inc_state = INC_ERROR;
+				if (session_get_error(SESSION(pop3_session)) == SESSION_ERROR_LOOKUP)
+					session->inc_state = INC_LOOKUP_ERROR;
 				break;
 			case SESSION_EOF:
 				session->inc_state = INC_EOF;
@@ -1029,7 +1215,6 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 {
 	gchar buf[BUFFSIZE];
 	Pop3Session *pop3_session = POP3_SESSION(inc_session->session);
-	gchar *total_size_str;
 	gint64 cur_total;
 	gint64 total;
 	gint cur_num;
@@ -1050,7 +1235,10 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 	if ((pop3_session->state == POP3_RETR ||
 	     pop3_session->state == POP3_RETR_RECV ||
 	     pop3_session->state == POP3_DELETE) && total_num_to_recv > 0) {
-		Xstrdup_a(total_size_str, to_human_readable(total), return);
+		gchar total_size_str[16];
+
+		to_human_readable_buf(total_size_str, sizeof(total_size_str),
+				      total);
 		g_snprintf(buf, sizeof(buf),
 			   _("Retrieving message (%d / %d) (%s / %s)"),
 			   cur_num, total_num_to_recv,
@@ -1079,8 +1267,7 @@ static void inc_progress_dialog_set_progress(IncProgressDialog *inc_dialog,
 		g_snprintf(buf, sizeof(buf),
 			   _("%d message(s) (%s) received"),
 			   pop3_session->cur_total_num,
-			   to_human_readable
-			   (pop3_session->cur_total_recv_bytes));
+			   to_human_readable(pop3_session->cur_total_recv_bytes));
 		progress_dialog_set_row_progress(inc_dialog->dialog,
 						 inc_dialog->cur_row, buf);
 	}
@@ -1263,6 +1450,17 @@ static gint inc_recv_message(Session *session, const gchar *msg, gpointer data)
 	return 0;
 }
 
+/**
+ * inc_drop_message:
+ * @session: Current Pop3Session.
+ * @file: Received message file.
+ * 
+ * Callback function to drop received message into local mailbox.
+ *
+ * Return value: DROP_OK if succeeded. DROP_ERROR if error occurred.
+ *   DROP_DONT_RECEIVE if the message should be skipped.
+ *   DROP_DELETE if the message should be deleted.
+ **/
 static gint inc_drop_message(Pop3Session *session, const gchar *file)
 {
 	FolderItem *inbox;
@@ -1273,11 +1471,14 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	gint val;
 	gboolean is_junk = FALSE;
 	gboolean is_counted = FALSE;
+	IncProgressDialog *inc_dialog;
 
 	g_return_val_if_fail(inc_session != NULL, DROP_ERROR);
 
 	gdk_threads_enter();
 
+	inc_dialog = (IncProgressDialog *)inc_session->data;
+
 	if (session->ac_prefs->inbox) {
 		inbox = folder_find_item_from_identifier
 			(session->ac_prefs->inbox);
@@ -1390,8 +1591,19 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	else {
 		val = DROP_OK;
 		if (!is_junk && is_counted &&
-		    fltinfo->actions[FLT_ACTION_MARK_READ] == FALSE)
+		    fltinfo->actions[FLT_ACTION_MARK_READ] == FALSE) {
 			inc_session->new_msgs++;
+
+			if (inc_dialog->result &&
+			    msginfo->subject && msginfo->fromname &&
+			    g_slist_length(inc_dialog->result->msg_summaries) < 5) {
+				IncMsgSummary *summary;
+				summary = g_new(IncMsgSummary, 1);
+				summary->subject = g_strdup(msginfo->subject);
+				summary->from = g_strdup(msginfo->fromname);
+				inc_dialog->result->msg_summaries = g_slist_append(inc_dialog->result->msg_summaries, summary);
+			}
+		}
 	}
 
 	procmsg_msginfo_free(msginfo);
@@ -1401,27 +1613,36 @@ static gint inc_drop_message(Pop3Session *session, const gchar *file)
 	return val;
 }
 
-static void inc_put_error(IncState istate, const gchar *msg)
+static void inc_put_error(IncSession *session, IncState istate, const gchar *pop3_msg)
 {
 	gchar *log_msg = NULL;
 	gchar *err_msg = NULL;
 	gboolean fatal_error = FALSE;
 
 	switch (istate) {
+	case INC_LOOKUP_ERROR:
+		log_msg = _("Server not found.");
+		if (prefs_common.no_recv_err_panel)
+			break;
+		err_msg = g_strdup_printf
+			(_("Server %s not found."), session->session->server);
+		break;
 	case INC_CONNECT_ERROR:
 		log_msg = _("Connection failed.");
 		if (prefs_common.no_recv_err_panel)
 			break;
-		err_msg = g_strdup(log_msg);
+		err_msg = g_strdup_printf
+			(_("Connection to %s:%d failed."),
+			 session->session->server, session->session->port);
 		break;
 	case INC_ERROR:
 		log_msg = _("Error occurred while processing mail.");
 		if (prefs_common.no_recv_err_panel)
 			break;
-		if (msg)
+		if (pop3_msg)
 			err_msg = g_strdup_printf
 				(_("Error occurred while processing mail:\n%s"),
-				 msg);
+				 pop3_msg);
 		else
 			err_msg = g_strdup(log_msg);
 		break;
@@ -1451,9 +1672,9 @@ static void inc_put_error(IncState istate, const gchar *msg)
 		log_msg = _("Mailbox is locked.");
 		if (prefs_common.no_recv_err_panel)
 			break;
-		if (msg)
+		if (pop3_msg)
 			err_msg = g_strdup_printf(_("Mailbox is locked:\n%s"),
-						  msg);
+						  pop3_msg);
 		else
 			err_msg = g_strdup(log_msg);
 		break;
@@ -1461,9 +1682,9 @@ static void inc_put_error(IncState istate, const gchar *msg)
 		log_msg = _("Authentication failed.");
 		if (prefs_common.no_recv_err_panel)
 			break;
-		if (msg)
+		if (pop3_msg)
 			err_msg = g_strdup_printf
-				(_("Authentication failed:\n%s"), msg);
+				(_("Authentication failed:\n%s"), pop3_msg);
 		else
 			err_msg = g_strdup(log_msg);
 		break;
@@ -1516,6 +1737,9 @@ gboolean inc_is_active(void)
 {
 	GList *cur;
 
+	if (inc_is_running)
+		return TRUE;
+
 	if (inc_dialog_list == NULL)
 		return FALSE;
 
