@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,6 +81,7 @@
 #include "socket.h"
 #include "stock_pixmap.h"
 #include "trayicon.h"
+#include "notificationwindow.h"
 #include "plugin.h"
 #include "plugin_manager.h"
 #include "foldersel.h"
@@ -132,6 +133,7 @@ static struct RemoteCmd {
 	GPtrArray *status_full_folders;
 	gchar *open_msg;
 	gboolean configdir;
+	gboolean safe_mode;
 	gboolean exit;
 	gboolean restart;
 	gchar *argv0;
@@ -224,6 +226,7 @@ int main(int argc, char *argv[])
 	GObject *syl_app;
 	PrefsAccount *new_account = NULL;
 	gboolean first_run = FALSE;
+	gchar *path;
 
 	app_init();
 	parse_cmd_opt(argc, argv);
@@ -294,6 +297,23 @@ int main(int argc, char *argv[])
 
 	gtkut_widget_init();
 
+	path = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, "icons", NULL);
+	if (is_dir_exist(path)) {
+		debug_print("icon theme dir: %s\n", path);
+		stock_pixbuf_set_theme_dir(path);
+	} else {
+		g_free(path);
+		if (g_path_is_absolute(THEMEDIR))
+			path = g_strconcat(THEMEDIR, NULL);
+		else
+			path = g_strconcat(get_startup_dir(), G_DIR_SEPARATOR_S, THEMEDIR, NULL);
+		if (is_dir_exist(path)) {
+			debug_print("icon theme dir: %s\n", path);
+			stock_pixbuf_set_theme_dir(path);
+		}
+	}
+	g_free(path);
+
 #ifdef G_OS_WIN32
 	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED_32, &icon);
 	iconlist = g_list_append(iconlist, icon);
@@ -607,6 +627,8 @@ static void parse_cmd_opt(int argc, char *argv[])
 					(argv[i + 1], -1, NULL, NULL, NULL);
 				i++;
 			}
+		} else if (!strncmp(argv[i], "--safe-mode", 11)) {
+			cmd.safe_mode = TRUE;
 		} else if (!strncmp(argv[i], "--exit", 6)) {
 			cmd.exit = TRUE;
 		} else if (!strncmp(argv[i], "--help", 6)) {
@@ -633,6 +655,7 @@ static void parse_cmd_opt(int argc, char *argv[])
 #endif
 			g_print("%s\n", _("  --exit                 exit Sylpheed"));
 			g_print("%s\n", _("  --debug                debug mode"));
+			g_print("%s\n", _("  --safe-mode            safe mode"));
 			g_print("%s\n", _("  --help                 display this help and exit"));
 			g_print("%s\n", _("  --version              output version information and exit"));
 
@@ -1273,6 +1296,12 @@ static void plugin_init(void)
 		return;
 	}
 
+	if (cmd.safe_mode) {
+		debug_print("plugin_init: safe mode enabled, skipping plug-in loading.\n");
+		STATUSBAR_POP(mainwin);
+		return;
+	}
+
 	ADD_SYM(prog_version);
 	ADD_SYM(app_will_exit);
 
@@ -1382,6 +1411,10 @@ static void plugin_init(void)
 	ADD_SYM(send_message_set_reply_flag);
 	ADD_SYM(send_message_set_forward_flags);
 
+	ADD_SYM(notification_window_open);
+	ADD_SYM(notification_window_set_message);
+	ADD_SYM(notification_window_close);
+
 	syl_plugin_signal_connect("plugin-load", G_CALLBACK(load_cb), NULL);
 
 	/* loading plug-ins from user plug-in directory */
@@ -1395,6 +1428,16 @@ static void plugin_init(void)
 			   NULL);
 	syl_plugin_load_all(path);
 	g_free(path);
+#elif defined(__APPLE__)
+	path = g_strconcat(get_startup_dir(), G_DIR_SEPARATOR_S,
+			   "Contents" G_DIR_SEPARATOR_S
+			   "Resources" G_DIR_SEPARATOR_S
+			   "lib" G_DIR_SEPARATOR_S
+			   "sylpheed" G_DIR_SEPARATOR_S
+			   PLUGIN_DIR,
+			   NULL);
+	syl_plugin_load_all(path);
+	g_free(path);
 #else
 	syl_plugin_load_all(PLUGINDIR);
 #endif
