@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2012 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2013 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -299,11 +299,15 @@ static void syl_plugin_class_init(SylPluginClass *klass)
 void syl_plugin_signal_connect(const gchar *name, GCallback callback,
 			       gpointer data)
 {
+	g_return_if_fail(plugin_obj != NULL);
+
 	g_signal_connect(plugin_obj, name, callback, data);
 }
 
 void syl_plugin_signal_disconnect(gpointer func, gpointer data)
 {
+	g_return_if_fail(plugin_obj != NULL);
+
 	g_signal_handlers_disconnect_by_func(plugin_obj, func, data);
 }
 
@@ -311,6 +315,8 @@ void syl_plugin_signal_emit(const gchar *name, ...)
 {
 	guint signal_id;
 
+	g_return_if_fail(plugin_obj != NULL);
+
 	if (g_signal_parse_name(name, G_TYPE_FROM_INSTANCE(plugin_obj), &signal_id, NULL, FALSE)) {
 		 \
 				va_list var_args;
@@ -345,6 +351,7 @@ gint syl_plugin_load(const gchar *name)
 	SylPluginLoadFunc load_func = NULL;
 	gchar *file;
 
+	g_return_val_if_fail(plugin_obj != NULL, -1);
 	g_return_val_if_fail(name != NULL, -1);
 
 	debug_print("syl_plugin_load: loading %s\n", name);
@@ -425,6 +432,8 @@ void syl_plugin_unload_all(void)
 {
 	GSList *cur;
 
+	g_return_if_fail(plugin_obj != NULL);
+
 	for (cur = module_list; cur != NULL; cur = cur->next) {
 		GModule *module = (GModule *)cur->data;
 		SylPluginUnloadFunc unload_func = NULL;
@@ -1364,3 +1373,30 @@ gint syl_plugin_send_message_set_forward_flags(const gchar *forward_targets)
 	GETFUNC("send_message_set_forward_flags");
 	return SAFE_CALL_ARG1_RET_VAL(func, forward_targets, -1);
 }
+
+gint syl_plugin_notification_window_open(const gchar *message,
+					 const gchar *submessage,
+					 guint timeout)
+{
+	gint (*func)(const gchar *, const gchar *, guint);
+
+	GETFUNC("notification_window_open");
+	return SAFE_CALL_ARG3_RET_VAL(func, message, submessage, timeout, -1);
+}
+
+void syl_plugin_notification_window_set_message(const gchar *message,
+						const gchar *submessage)
+{
+	void (*func)(const gchar *, const gchar *);
+
+	GETFUNC("notification_window_set_message");
+	SAFE_CALL_ARG2(func, message, submessage);
+}
+
+void syl_plugin_notification_window_close(void)
+{
+	void (*func)(void);
+
+	GETFUNC("notification_window_close");
+	SAFE_CALL(func);
+}
