@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2012 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,7 @@
 #include "stock_pixmap.h"
 #include "alertpanel.h"
 #include "folder.h"
+#include "plugin.h"
 
 static struct FilterRuleEditWindow {
 	GtkWidget *window;
@@ -171,6 +172,9 @@ FilterRule *prefs_filter_edit_open(FilterRule *rule, const gchar *header,
 	GTK_EVENTS_FLUSH();
 	gtk_widget_show(rule_edit_window.window);
 
+	syl_plugin_signal_emit("prefs-filter-edit-open", rule, header, key,
+			       rule_edit_window.window);
+
 	rule_edit_window.new_rule = NULL;
 	rule_edit_window.edit_finished = FALSE;
 	while (rule_edit_window.edit_finished == FALSE)
@@ -273,6 +277,7 @@ static void prefs_filter_edit_create(void)
 	MENUITEM_ADD(menu, menuitem,
 		     _("If all of the following conditions match"), FLT_AND);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(bool_op_optmenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(bool_op_optmenu), FLT_AND);
 
 	cond_scrolled_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(cond_scrolled_win);
@@ -369,7 +374,7 @@ static void prefs_filter_edit_clear(void)
 static void prefs_filter_edit_rule_to_dialog(FilterRule *rule,
 					     const gchar *default_name)
 {
-	gint index = 0;
+	gint index = 1;
 	static gint count = 1;
 
 	if (rule && rule->name)
@@ -390,7 +395,7 @@ static void prefs_filter_edit_rule_to_dialog(FilterRule *rule,
 			(GTK_OPTION_MENU(rule_edit_window.bool_op_optmenu),
 			 GINT_TO_POINTER(rule->bool_op), NULL);
 		if (index < 0)
-			index = 0;
+			index = 1;
 	}
 	if (index >= 0) {
 		gtk_option_menu_set_history
@@ -563,8 +568,8 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 
 	menu = gtk_menu_new();
 	gtk_widget_show(menu);
-	MENUITEM_ADD(menu, menuitem, _("is longer than"), PF_AGE_LONGER);
 	MENUITEM_ADD(menu, menuitem, _("is shorter than"), PF_AGE_SHORTER);
+	MENUITEM_ADD(menu, menuitem, _("is longer than"), PF_AGE_LONGER);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(age_match_optmenu), menu);
 
 	status_match_optmenu = gtk_option_menu_new();
@@ -583,7 +588,7 @@ CondHBox *prefs_filter_edit_cond_hbox_create(FilterCondEdit *cond_edit)
 	gtk_widget_show(key_entry);
 	gtk_box_pack_start(GTK_BOX(hbox), key_entry, TRUE, TRUE, 0);
 
-	spin_btn_adj = gtk_adjustment_new(0, 0, 99999, 1, 10, 100);
+	spin_btn_adj = gtk_adjustment_new(0, 0, 99999, 1, 10, 0);
 	spin_btn = gtk_spin_button_new(GTK_ADJUSTMENT(spin_btn_adj), 1, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), spin_btn, FALSE, FALSE, 0);
 	gtk_widget_set_size_request(spin_btn, 64, -1);
@@ -788,7 +793,7 @@ void prefs_filter_edit_cond_hbox_set(CondHBox *hbox, FilterCond *cond)
 	CondMenuType cond_type = PF_COND_NONE;
 	MatchMenuType match_type = PF_MATCH_NONE;
 	SizeMatchType size_type = PF_SIZE_LARGER;
-	AgeMatchType age_type = PF_AGE_LONGER;
+	AgeMatchType age_type = PF_AGE_SHORTER;
 	StatusMatchType status_type = PF_STATUS_MATCH;
 
 	switch (cond->type) {
@@ -1845,6 +1850,24 @@ FilterCond *prefs_filter_edit_cond_hbox_to_cond(CondHBox *hbox,
 	return cond;
 }
 
+static gboolean check_dest_folder(const gchar *dest, gchar **error_msg)
+{
+	FolderItem *item;
+
+	if (!dest || *dest == '\0') {
+		*error_msg = _("Destination folder is not specified.");
+		return FALSE;
+	}
+
+	item = folder_find_item_from_identifier(dest);
+	if (!item || !item->path || !item->parent) {
+		*error_msg = _("The specified destination folder does not exist.");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 FilterAction *prefs_filter_edit_action_hbox_to_action(ActionHBox *hbox,
 						      gchar **error_msg)
 {
@@ -1859,18 +1882,13 @@ FilterAction *prefs_filter_edit_action_hbox_to_action(ActionHBox *hbox,
 	switch (action_menu_type) {
 	case PF_ACTION_MOVE:
 		str = gtk_entry_get_text(GTK_ENTRY(hbox->folder_entry));
-		if (str && *str)
-			action = filter_action_new(FLT_ACTION_MOVE,
-						   str);
-		else
-			error_msg_ = _("Destination folder is not specified.");
+		if (check_dest_folder(str, &error_msg_))
+			action = filter_action_new(FLT_ACTION_MOVE, str);
 		break;
 	case PF_ACTION_COPY:
 		str = gtk_entry_get_text(GTK_ENTRY(hbox->folder_entry));
-		if (str && *str)
+		if (check_dest_folder(str, &error_msg_))
 			action = filter_action_new(FLT_ACTION_COPY, str);
-		else
-			error_msg_ = _("Destination folder is not specified.");
 		break;
 	case PF_ACTION_NOT_RECEIVE:
 		action = filter_action_new(FLT_ACTION_NOT_RECEIVE, NULL);
@@ -1955,7 +1973,7 @@ static FilterRule *prefs_filter_edit_dialog_to_rule(void)
 	FilterRule *rule = NULL;
 	GSList *cur;
 	const gchar *rule_name;
-	FilterBoolOp bool_op = FLT_OR;
+	FilterBoolOp bool_op;
 	GSList *cond_list = NULL;
 	GSList *action_list = NULL;
 	GtkWidget *bool_op_menuitem;
