@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2009 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -278,6 +278,7 @@ static void query_search_create(void)
 	MENUITEM_ADD(bool_menu, menuitem, _("Match all of the following"),
 		     FLT_AND);
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(bool_optmenu), bool_menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(bool_optmenu), FLT_AND);
 
 	hbbox = gtk_hbutton_box_new();
 	gtk_widget_show(hbbox);
@@ -399,6 +400,9 @@ static void query_search_create(void)
 
 	APPEND_COLUMN(_("Folder"), COL_FOLDER, 0);
 	APPEND_COLUMN(_("Subject"), COL_SUBJECT, 200);
+#if GTK_CHECK_VERSION(2, 14, 0)
+	gtk_tree_view_column_set_expand(column, TRUE);
+#endif
 	APPEND_COLUMN(_("From"), COL_FROM, 180);
 	APPEND_COLUMN(_("Date"), COL_DATE, 0);
 
@@ -463,7 +467,7 @@ static FilterRule *query_search_dialog_to_rule(const gchar *name,
 {
 	const gchar *id;
 	FolderItem *item_;
-	FilterBoolOp bool_op = FLT_OR;
+	FilterBoolOp bool_op;
 	gboolean recursive;
 	gboolean case_sens;
 	GSList *cond_list;
