@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2011 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2014 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,7 +117,7 @@ static const gchar *ui_def =
 	"      <menuitem name='Open' action='OpenAction'/>"
 	"      <menuitem name='Delete' action='DeleteAction'/>"
 	"      <separator />"
-	"      <menuitem name='Update' action='UpdateAction'/>"
+	"      <menuitem name='Resume' action='ResumeAction'/>"
 	"      <menuitem name='Stop' action='StopAction'/>"
 	"      <separator />"
 	"      <menuitem name='Close' action='CloseAction'/>"
@@ -225,7 +225,7 @@ static void rpop3_close		(GtkButton	*button,
 static void rpop3_recv_cb	(void);
 static void rpop3_open_cb	(void);
 static void rpop3_delete_cb	(void);
-static void rpop3_update_cb	(void);
+static void rpop3_resume_cb	(void);
 static void rpop3_stop_cb	(void);
 static void rpop3_close_cb	(void);
 static void rpop3_about_cb	(void);
@@ -248,7 +248,7 @@ static GtkActionEntry action_entries[] = {
 	{"ReceiveAction", NULL, N_("_Get"), "<Control>G", NULL, rpop3_recv_cb},
 	{"OpenAction", GTK_STOCK_OPEN, NULL, NULL, NULL, rpop3_open_cb},
 	{"DeleteAction", GTK_STOCK_DELETE, NULL, "<Shift>Delete", NULL, rpop3_delete_cb},
-	{"UpdateAction", GTK_STOCK_REFRESH, NULL, NULL, NULL, rpop3_update_cb},
+	{"ResumeAction", GTK_STOCK_REFRESH, N_("_Resume"), NULL, NULL, rpop3_resume_cb},
 	{"StopAction", GTK_STOCK_STOP, NULL, NULL, NULL, rpop3_stop_cb},
 	{"CloseAction", GTK_STOCK_CLOSE, NULL, NULL, NULL, rpop3_close_cb},
 	{"HelpAction", NULL, N_("_Help"), NULL, NULL, NULL},
@@ -432,6 +432,9 @@ static void rpop3_window_create(PrefsAccount *account)
 
         APPEND_COLUMN(_("No."), COL_NUMBER, 0);
         APPEND_COLUMN(_("Subject"), COL_SUBJECT, 200);
+#if GTK_CHECK_VERSION(2, 14, 0)
+	gtk_tree_view_column_set_expand(column, TRUE);
+#endif
         APPEND_COLUMN(_("From"), COL_FROM, 160);
         APPEND_COLUMN(_("Date"), COL_DATE, 0);
         APPEND_COLUMN(_("Size"), COL_SIZE, 0);
@@ -473,7 +476,7 @@ static void rpop3_window_create(PrefsAccount *account)
 	g_object_set(action, "sensitive", FALSE, NULL);
 	rpop3_window.delete_action = action;
 
-	action = gtk_ui_manager_get_action(ui, "/RPop3Menu/File/Update");
+	action = gtk_ui_manager_get_action(ui, "/RPop3Menu/File/Resume");
 	g_object_set(action, "sensitive", FALSE, NULL);
 	rpop3_window.update_action = action;
 
@@ -845,11 +848,22 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 	    pop3_session->state != POP3_GETSIZE_LIST_RECV) {
 		val = pop3_ok(pop3_session, msg);
 		if (val != PS_SUCCESS) {
-			if (val != PS_NOTSUPPORTED) {
+			if (val == PS_SOCKET) {
 				pop3_session->state = POP3_ERROR;
 				gdk_threads_leave();
 				return -1;
 			}
+			if (val != PS_NOTSUPPORTED) {
+				if (pop3_session->state != POP3_LOGOUT) {
+					if (pop3_logout_send(pop3_session) == PS_SUCCESS) {
+						gdk_threads_leave();
+						return 0;
+					} else {
+						gdk_threads_leave();
+						return -1;
+					}
+				}
+			}
 		}
 
 		if (*body == '+' || *body == '-')
@@ -957,7 +971,10 @@ static gint rpop3_session_recv_msg(Session *session, const gchar *msg)
 		rpop3_idle(TRUE);
 		break;
 	case POP3_LOGOUT:
-		pop3_session->state = POP3_DONE;
+		if (val == PS_SUCCESS)
+			pop3_session->state = POP3_DONE;
+		else
+			pop3_session->state = POP3_ERROR;
 		session_disconnect(session);
 		break;
 	case POP3_ERROR:
@@ -989,7 +1006,7 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
                 if (val == PS_SUCCESS) {
 			if (rpop3_window.cancelled) {
 				rpop3_status_label_set(_("Quitting..."));
-				pop3_logout_send(rpop3_window.session);
+				pop3_logout_send(pop3_session);
 			} else
 				pop3_getsize_list_send(pop3_session);
 		} else
@@ -1001,7 +1018,7 @@ static gint rpop3_session_recv_data_finished(Session *session, guchar *data,
 			pop3_session->cur_msg = 1;
 			if (rpop3_window.cancelled || pop3_session->count == 0) {
 				rpop3_status_label_set(_("Quitting..."));
-				pop3_logout_send(rpop3_window.session);
+				pop3_logout_send(pop3_session);
 			} else {
 				pop3_session->cur_msg = pop3_session->count;
 				gtk_widget_set_sensitive(rpop3_window.stop_btn,
@@ -1062,7 +1079,7 @@ static gint rpop3_session_recv_data_as_file_finished(Session *session,
 		if (rpop3_top_recv(pop3_session, fp, len) == PS_SUCCESS) {
 			if (rpop3_window.cancelled) {
 				rpop3_status_label_set(_("Quitting..."));
-				pop3_logout_send(rpop3_window.session);
+				pop3_logout_send(pop3_session);
 			} else if (!rpop3_window.stop_load &&
 				 (pop3_session->cur_msg > 1)) {
 				pop3_session->cur_msg--;
@@ -1084,7 +1101,8 @@ static gint rpop3_session_recv_data_as_file_finished(Session *session,
 					(rpop3_window.delete_btn, TRUE);
 				if (pop3_session->cur_msg == 1)
 					gtk_widget_set_sensitive(rpop3_window.stop_btn, FALSE);
-				gtk_button_set_label(GTK_BUTTON(rpop3_window.stop_btn), GTK_STOCK_REFRESH);
+				else
+					gtk_button_set_label(GTK_BUTTON(rpop3_window.stop_btn), _("_Resume"));
 				g_object_set(rpop3_window.recv_action,
 					     "sensitive", TRUE, NULL);
 				g_object_set(rpop3_window.open_action,
@@ -1278,7 +1296,10 @@ static void rpop3_read_next(GtkButton *button, gpointer data)
 	rpop3_window.session->cur_msg = i;
 
 	rpop3_window.stop_load = FALSE;
-	
+
+	gtk_widget_set_sensitive(rpop3_window.recv_btn, FALSE);
+	gtk_widget_set_sensitive(rpop3_window.open_btn, FALSE);
+	gtk_widget_set_sensitive(rpop3_window.delete_btn, FALSE);
 	gtk_button_set_label(GTK_BUTTON(rpop3_window.stop_btn), GTK_STOCK_STOP);
 	gtk_widget_set_sensitive(rpop3_window.stop_btn, TRUE);
 	g_object_set(rpop3_window.update_action, "sensitive", FALSE, NULL);
@@ -1323,7 +1344,7 @@ static void rpop3_delete_cb(void)
 	rpop3_delete(NULL, NULL);
 }
 
-static void rpop3_update_cb(void)
+static void rpop3_resume_cb(void)
 {
 	rpop3_stop(NULL, NULL);
 }
